#!/usr/bin/python
#backup_monitor.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#This does a bunch of things.  
#1)  monitor the lists of file sent back from suppliers,
#    if there is a gap we need to try to fix it
#    * main class is _BackupMonitor,
#      it saves the lists of files in _BackupListFiles,
#      breaks down the lists of files into info on a single backup in _SupplierBackupInfo
#    * _BlockRebuilder takes care of a single broken block,
#      request what we have available, builds whatever we can
#      and stops either when we have fixed everything
#      or there is nothing more we can do
#    * _BlockRebuilder requests files through io_throttle and sends out the fixed files
#      also through io_throttle
#
#2)  if a backup is unfixable, not enough information, we delete it CleanupBackups in _BackupMonitor
#
#3)  every hour it requests a list of files from each supplier - _hourlyRequestListFiles
#
#4)  every hour it tests a file from each supplier,
#    seeing if they have the data they claim,
#    and that it is correct
#    * data is stored in _SuppliersSet and _SupplierRemoteTestResults,
#    was data good, bad, being rebuilt, or they weren't online 
#    and we got no data on the result
#    * if a supplier hasn't been seen in settings.FireInactiveSupplierIntervalHours()
#    we replace them
#
#
# Strategy for automatic backups
#
# 1) Full backup to alternating set of nodes every 2 weeks.
#
# 2) Full monthly, then incremental weekly and daily
#
# 3) One time full and then incremental monthly, weekly
#
# 4) Break alphabetical list of files into N parts and do full
#    backup on one of those and incrementals on the rest.
#    So we no longer need part of the incremental history
#      every time, and after N times older stuff can toss.
#      so every day is part full and part incremental.  Cool.
#
#  Want user to be able to specify what he wants, or at least
#  select from a few reasonable choices.
#
#
# May just do (1) to start with.
#
# This code also wakes up every day and fires off localtester, remotetester
#   on some reasonable random stuff.
#
# This manages the whole thing, so after GUI, this has the highest level control functions.
#
# Some competitors let people choose what days to backup on.  Not sure this
#   is really so great though, once we have incremental.
#
# Some can handle partial file changes.  So if a 500 MB mail file only has
# a little bit appended, only the little bit is backed up.
#
# Some competitors can turn the computer off after a backup, but
# we need it to stay on for P2P stuff.
#
# need to record if zip, tar, dump, etc
#
# If we do regular full backups often, then we might not bother scrubbing older stuff.
# Could reduce the bandwidth needed (since scrubbing could use alot.l


import os
import sys
import time
import random
import gc


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in backup_monitor.py')

from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet import threads


import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.transport_control as transport_control
import lib.settings as settings
import lib.contacts as contacts
import lib.eccmap as eccmap
import lib.tmpfile as tmpfile
import lib.diskspace as diskspace
import lib.automat as automat
import lib.automats as automats


import backup_rebuilder
import fire_hire
import list_files_orator 

import backups
import customerservice
import io_throttle
import backup_db
import contacts_status

_BackupMonitor = None

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _BackupMonitor
    if _BackupMonitor is None:
        _BackupMonitor = BackupMonitor('backup_monitor', 'READY', 4)
    if event is not None:
        _BackupMonitor.automat(event, arg)
    return _BackupMonitor


class BackupMonitor(automat.Automat):
    timers = {'timer-1sec':   (1,       ['RESTART']), 
              'timer-1hour':  (60*60,   ['READY']), }

    def state_changed(self, oldstate, newstate):
        automats.set_global_state('MONITOR ' + newstate)

    def A(self, event, arg):
        #---READY---
        if self.state == 'READY':
            if event == 'restart' or event == 'timer-1hour':
                self.state = 'RESTART'
        #---RESTART---
        elif self.state == 'RESTART':
            if event == 'timer-1sec' and backup_rebuilder.A().state in [ 'STOPPED' , 'DONE' ]:
                list_files_orator.A('need-files')
                self.state = 'LIST_FILES'
        #---LIST_FILES---
        elif self.state == 'LIST_FILES':
            if event == 'list_files_orator.state' and arg is 'SAW_FILES':
                self.doPrepareListBackups()
                self.state = 'LIST_BACKUPS'
            elif event == 'list_files_orator.state' and arg is 'NO_FILES':
                self.state = 'READY'
            elif event == 'restart':
                self.state = 'RESTART'
        #---LIST_BACKUPS---
        elif self.state == 'LIST_BACKUPS':
            if event == 'list-backups-done':
                backup_rebuilder.A('start')
                self.state = 'REBUILDING'
            elif event == 'restart':
                self.state = 'RESTART'
        #---REBUILDING---
        elif self.state == 'REBUILDING':
            if event == 'backup_rebuilder.state' and arg is 'DONE':
                fire_hire.A('start')
                self.state = 'FIRE_HIRE'
            elif event == 'backup_rebuilder.state' and arg is 'STOPPED':
                self.state = 'READY'
            elif event == 'restart':
                backup_rebuilder.SetStoppedFlag()
                self.state = 'RESTART'
        #---FIRE_HIRE---
        elif self.state == 'FIRE_HIRE':
            if event == 'hire-new-supplier' or event == 'restart': # or ( event == 'fire-hire-finished' and self.isFailedSuppliers() ):
                self.state = 'RESTART'
            elif event == 'fire-hire-finished': # and not self.isFailedSuppliers():
                self.doCleanUpBackups()
                self.state = 'READY'

    def doPrepareListBackups(self):
        # take remote and local backups and get union from it 
        allBackupIDs = set(backups.local_files().keys() + backups.remote_files().keys())
        # take only backups from data base
        allBackupIDs.intersection_update(backup_db.GetBackupIds())
        # remove running backups
        allBackupIDs.difference_update(backups.backups_in_process())
        # sort it in reverse order - newer backups should be repaired first
        allBackupIDs = misc.sorted_backup_ids(list(allBackupIDs), True)
        # add backups to the queue
        backup_rebuilder.AddBackupsToWork(allBackupIDs)
        # update suppliers, clear stats for all 
        backups.suppliers_set().UpdateSuppliers(contacts.getSupplierIDs()) 
        dhnio.Dprint(6, 'backup_monitor.doPrepareListBackups %s' % str(allBackupIDs))
        self.automat('list-backups-done')

    def doCleanUpBackups(self):
        backupsToKeep = settings.getGeneralBackupsToKeep()
        bytesUsed = backup_db.GetTotalBackupsSize()
        bytesNeeded = diskspace.GetBytesFromString(settings.getCentralMegabytesNeeded()) 
        dhnio.Dprint(6, 'backup_monitor.doCleanUpBackups backupsToKeep=%dm used=%d needed=%d' % (
            backupsToKeep, bytesUsed, bytesNeeded))
        if backupsToKeep > 0:
            for backupDir in backup_db.GetBackupDirectories():
                backupIDsForDirectory = backup_db.GetDirBackupIds(backupDir)
                backupIDsForDirectory.reverse()
                while len(backupIDsForDirectory) > backupsToKeep:
                    backupID = backupIDsForDirectory.pop(0)
                    dhnio.Dprint(6, 'backup_monitor.doCleanUpBackups %d of %d backups for %s, so remove %s' % (
                        len(backupIDsForDirectory), backupsToKeep, backupDir, backupID))
                    backup_db.AbortRunningBackup(backupID)
                    backups.DeleteBackup(backupID)
        # we need to fit used space into needed space (given from other users)
        # they trust us - do not need to take extra space from our friends
        # so remove oldest backups, but keep at least one for every folder - at least locally!
        # still our suppliers will remove our "extra" files by their "local_tester" 
        if bytesNeeded <= bytesUsed:
            all_backups = backup_db.GetBackupIds(full_info_please=True)
            all_backups.reverse()
            for i in range(len(all_backups)-1, -1, -1):
                backupID = all_backups[i][0]
                backupDir = all_backups[i][1]
                backupSize = all_backups[i][2]
                #if len(backup_db.BackupDirs().get(backupDir, [])) <= 1:
                if len(backup_db.GetDirBackupIds(backupDir)) <= 1:
                    continue
                dhnio.Dprint(6, 'backup_monitor.doCleanUpBackups over use %d of %d, so remove %s of %s' % (
                    bytesUsed, bytesNeeded, backupID, backupDir))
                backup_db.AbortRunningBackup(backupID)
                backups.DeleteBackup(backupID)
                bytesUsed -= backupSize
                if bytesNeeded > bytesUsed:
                    break 
        collected = gc.collect()
        dhnio.Dprint(6, 'backup_monitor.doCleanUpBackups collected %d objects' % collected)
                

def Restart():
    dhnio.Dprint(4, 'backup_monitor.Restart')
    A('restart')


def shutdown():
    dhnio.Dprint(4, 'backup_monitor.shutdown')
    automat.clear_object(A().index)

        

#    def CleanupBackups(self):
#        #TODO - track if we've heard from enough suppliers to know a backup doesn't exist
#        backupsToKeep = int(settings.getGeneralBackupsToKeep())
#        dhnio.Dprint(12+self.dprintAdjust, "backup_monitor.CleanupBackups, keep " + str(backupsToKeep) + " backups")
#        foundBackups = 0 # how many backups for a directory we've found
#        didDelete = False
#        delay = 1
#        #potentially called many times after a request list files all
#        if time.time() - self.backupDBCheck > 60.0: 
#            # if it has been longer than x seconds since the last time we ran this
#            self.backupDBCheck = time.time()
#            backupDirectories = backup_db.GetBackupDirectories()
#            for backupDir in backupDirectories:
#                foundBackups = 0
#                backupIDsForDirectory = backup_db.GetDirBackupIds(backupDir)
#                if len(backupIDsForDirectory) > 0:
#                    backupIDsForDirectory.reverse()
#                    for backupId in backupIDsForDirectory:
#                        if backupId not in backups_in_process():
#                            if self.backupListFiles.has_key(str(backupId)):
#                                foundBackups = foundBackups + 1
#                                if backupsToKeep > 0 and foundBackups > backupsToKeep:
#                                    dhnio.Dprint(2+self.dprintAdjust, "backup_monitor.CleanupBackups delete backup " + str(backupId) + " for directory " + str(backupDir) + " beyond the " + str(backupsToKeep) + " backups to keep")
#                                    self.DeleteBackup(backupId)
#                                    #customerservice.DeleteWholeBackup(backupId)
#                                    didDelete = True
#                            else: 
#                                # so we have something in our list of backups 
#                                # that should have happened, but no suppliers have info on it
#                                if len(self.rawListFiles) > self.eccMap.CorrectableErrors:
#                                    dhnio.Dprint(2+self.dprintAdjust, "backup_monitor.CleanupBackups too many suppliers don't have info on backup " + backupId + ", missing " + str(len(self.rawListFiles)) + " correctable " + str(self.eccMap.CorrectableErrors))
#                                    #   we temporary remove this feature
#                                    #   because something going wrong
#                                    #   backups are deleted after finishing
#                                    #backup_db.DeleteDirBackup(backupId)
#                                    #customerservice.DeleteWholeBackup(backupId) 
#                                    # may not be necessary if all suppliers don't have the backup
#                                    didDelete = True
#            if didDelete:
#                dhnio.Dprint(12+self.dprintAdjust, "backup_monitor.CleanupBackups scheduling another listfilesall at " + str(time.asctime(time.localtime(time.time()))))
#                reactor.callLater(30,customerservice.RequestListFilesAll)




from forms import GetParam
from forms.input import FormControlMixin

import wx
import wx.combo


class SingleSelectControl(wx.Choice, FormControlMixin):
    def __init__(self, parent, form, tag, parser, optionList, **kwargs):
        FormControlMixin.__init__(self, form, tag)
        self.values = []
        contents = []
        selection = 0
        for idx, option in enumerate(optionList):
            contents.append(parser.GetSource()[option.GetBeginPos():option.GetEndPos1()])
            self.values.append(GetParam(option, 'VALUE', ''))
            if option.HasParam("SELECTED") and not selection:
                selection = idx
        wx.Choice.__init__(self, parent,
            choices = contents,)
        self.SetSelection(selection)

    def GetValue(self):
        sel = self.GetSelection()
        value = self.values[sel]
        return value





#!/usr/bin/python
#transport_udp.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import time


from twisted.internet import reactor
from twisted.internet.task import LoopingCall 
from twisted.internet.defer import Deferred, succeed


import dhnio
import misc
import settings
import nameurl
import contacts
import identitycache

import stun
import shtoom.stun 
import shtoom.nat
                                                                 
import automat
import transport_udp_session 
import transport_udp_server     

_TransportUDP = None
_IsServer = False

#------------------------------------------------------------------------------ 

def init(client,):
    A('init', client)
    

def shutdown():
    A('stop')
    
# fast=True: put filename in the top of sending queue,
# fast=False: append to the bottom of the queue    
def send(filename, host, port, fast=False):
    # dhnio.Dprint(6, "transport_udp.send %s to %s:%s" % (os.path.basename(filename), host, port))
    A('send-file', ((host, int(port)), (filename, fast)))

def getListener():
    return A()

#------------------------------------------------------------------------------ 

def Start():
    A('start')
    
    
def Stop():
    A('stop')


def ListContactsCallback(oldlist, newlist):
    A('list-contacts', (oldlist, newlist))
    

def SendStatusCallback(host, filename, status, proto='', error=None, message=''):
    try:
        from transport_control import sendStatusReport
        sendStatusReport(host, filename, status, proto, error, message)
    except:
        dhnio.DprintException()


def ReceiveStatusCallback(filename, status, proto='', host=None, error=None, message=''):
    try:
        from transport_control import receiveStatusReport
        receiveStatusReport(filename, status, proto, host, error, message)
    except:
        dhnio.DprintException()

def SendControlFunc(prev_read_size, chunk_size):
    try:
        from transport_control import SendTrafficControl
        return SendTrafficControl(prev_read_size, chunk_size)
    except:
        dhnio.DprintException()
        return chunk_size
        
def ReceiveControlFunc(new_data_size):
    try:
        from transport_control import ReceiveTrafficControl
        return ReceiveTrafficControl(new_data_size)
    except:
        dhnio.DprintException()
        return 0

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _TransportUDP
    if _TransportUDP is None:
        _TransportUDP = TransportUDP()
    if event is not None:
        _TransportUDP.automat(event, arg)
    return _TransportUDP


class TransportUDP(automat.Automat):
    def __init__(self):
        self.client = None
        # self.index_by_idurl = {}
        automat.Automat.__init__(self, 'transport_udp', 'STOPPED', 6)
        
    def A(self, event, arg):
        #---STARTED---
        if self.state is 'STARTED':
            # if event == 'datagram-received':
            #     self.doSessionDataReceive(arg)
            if event == 'send-file':
                self.doClientOutboxFile(arg)
            elif event == 'stop':
                self.doShutDownAllUDPSessions()
                self.doShutdownClient()
                self.state = 'STOPPED'
            elif event == 'list-contacts' and self.isNewUDPContact(arg):
                self.doRemoveOldUDPSession(arg)
                self.doCreateNewUDPSession(arg)
            elif event == 'ip-port-changed':
                self.doRestartUDPSession(arg)
        #---STOPPED---
        elif self.state is 'STOPPED':
            if event == 'init' and self.isOpenedIP(arg):
                self.doInitServer(arg)
                self.state = 'SERVER'
            elif event == 'init' and not self.isOpenedIP(arg):
                self.doInitClient(arg)
                self.state = 'CLIENT'
        #---SERVER---
        elif self.state is 'SERVER':
            # if event == 'datagram-received':
            #     self.doServerDataReceive(arg)
            if event == 'send-file':
                self.doServerOutboxFile(arg)
            elif event == 'stop':
                self.state = 'STOPPED'
        #---CLIENT---
        elif self.state is 'CLIENT':
            if event == 'start':
                self.doStartAllUDPSessions()
                self.state = 'STARTED'
            elif event == 'stop':
                self.doShutdownClient()
                self.state = 'STOPPED'

    def isOpenedIP(self, arg):
        if not arg:
            return False
        if not arg.externalAddress:
            return False
        if not arg.localAddress:
            return False            
        return arg.externalAddress[0] == arg.localAddress

    def isNewUDPContact(self, arg):
        return arg[0] != arg[1]
    
    def doInitServer(self, arg):
        self.client = arg
        transport_udp_server.init(self.client)
        # self.client.datagram_received_callback = self.datagramReceived
        self.client.datagram_received_callback = transport_udp_server.protocol().datagramReceived
        transport_udp_server.SetReceiveStatusCallback(ReceiveStatusCallback)
        transport_udp_server.SetSendStatusCallback(SendStatusCallback)
        transport_udp_server.SetReceiveControlFunc(ReceiveControlFunc)
        transport_udp_server.SetSendControlFunc(SendControlFunc)
        
    def doInitClient(self, arg):
        self.client = arg
        # self.client.datagram_received_callback = self.datagramReceived
        self.client.datagram_received_callback = transport_udp_session.data_received
        transport_udp_session.init(self)
        transport_udp_session.SetReceiveStatusCallback(ReceiveStatusCallback)
        transport_udp_session.SetSendStatusCallback(SendStatusCallback)
        transport_udp_session.SetReceiveControlFunc(ReceiveControlFunc)
        transport_udp_session.SetSendControlFunc(SendControlFunc)
        transport_udp_session.StartTimers()
        
    def doShutdownClient(self):
        transport_udp_session.StopTimers()
    
    def doSessionDataReceive(self, arg):
        address = arg[1]
#        s = transport_udp_session.sessions().get(address, None)
#        if s is None:
#            transport_udp_session.A(address, 'init', None)
        transport_udp_session.A(address, 'datagram-received', arg)
        
    def doServerDataReceive(self, arg):
        p = transport_udp_server.protocol() 
        if p:
            p.datagramReceived(arg[0], arg[1])
    
    def doStartAllUDPSessions(self):
        all = contacts.getContactsAndCorrespondents()
        all.append(settings.CentralID())
        # all.remove(misc.getLocalID())
        for idurl in all:
            ident = contacts.getContact(idurl)
            if ident is None:
                continue
            udp_contact = ident.getProtoContact('udp')
            if udp_contact is None:
                continue
            try:
                proto, host, port, filename = nameurl.UrlParse(udp_contact)
                udp_contact = (host, int(port))
            except:
                dhnio.DprintException()
                continue
            transport_udp_session.A(udp_contact, 'init', idurl)
            # self.index_by_idurl[idurl] = udp_contact
        
    def doShutDownAllUDPSessions(self):
        transport_udp_session.shutdown_all()
        # self.index_by_idurl.clear()
    
    def doCreateNewUDPSession(self, arg):
        dhnio.Dprint(6, 'transport_udp.doRemoveOldUDPSession')
        for idurl in arg[1]:
            if idurl == misc.getLocalID():
                continue
            if idurl not in arg[0]:
                ident = contacts.getContact(idurl)
                if ident is None:
                    continue
                udp_contact = ident.getProtoContact('udp')
                if udp_contact is None:
                    continue
                try:
                    proto, host, port, filename = nameurl.UrlParse(udp_contact)
                    udp_contact = (host, int(port))
                except:
                    dhnio.DprintException()
                    continue
                dhnio.Dprint(6, 'transport_udp.doCreateNewUDPSession with %s [%s]' % (nameurl.GetName(idurl), str(udp_contact)))
                transport_udp_session.A(udp_contact, 'init', idurl)
                # self.index_by_idurl[idurl] = udp_contact
    
    def doRemoveOldUDPSession(self, arg):
        dhnio.Dprint(6, 'transport_udp.doRemoveOldUDPSession')
        for idurl in arg[0]:
            if idurl == misc.getLocalID():
                continue
            if idurl not in arg[1]:
                ident = contacts.getContact(idurl)
                if ident is None:
                    continue
                udp_contact = ident.getProtoContact('udp')
                if udp_contact is None:
                    continue
                try:
                    ip, port = udp_contact[6:].split(':')
                    udp_contact = (ip, int(port))
                except:
                    dhnio.DprintException()
                    continue
                if udp_contact in transport_udp_session.sessions():
                    dhnio.Dprint(6, 'transport_udp.doRemoveOldUDPSession with %s [%s]' % (nameurl.GetName(idurl), str(udp_contact)))
                    transport_udp_session.A(udp_contact, 'shutdown')
                    # self.index_by_idurl.pop(idurl, None)
                
    def doRestartUDPSession(self, arg):
        idurl = arg
        ident = contacts.getContact(arg)
        if ident is None:
            return
        udp_contact = ident.getProtoContact('udp')
        if udp_contact is None:
            return
        try:
            ip, port = udp_contact[6:].split(':')
            udp_contact = (ip, int(port))
        except:
            dhnio.DprintException()
            return
        dhnio.Dprint(6, 'transport_udp.doRestartUDPSession with %s [%s]' % (nameurl.GetName(idurl), str(udp_contact)))
        transport_udp_session.A(udp_contact, 'init', idurl)
        # self.index_by_idurl[idurl] = udp_contact
                
    def doServerOutboxFile(self, arg):
        transport_udp_server.send(arg[0], arg[1][0], arg[1][1])    
        
    def doClientOutboxFile(self, arg):
        transport_udp_session.outbox_file(arg[0], arg[1][0], arg[1][1])
    
    def stopListening(self):
        dhnio.Dprint(4, 'transport_udp.stopListening')
        self.automat('stop')
        res = stun.stopUDPListener()
        if res is None:
            res = succeed(1)
        return res

    def datagramReceived(self, datagram, address):
        self.event('datagram-received', (datagram, address))
#        if self.state == 'CLIENT':
#            transport_udp_session.data_received(datagram, address, self)
#        elif self.state == 'SERVER':
#            transport_udp_server.protocol().datagramReceived(datagram, addr)
#        else:
#            # TO DO
#            pass

#------------------------------------------------------------------------------ 

def main():
    sys.path.append('..')

    def _go_stun(port):
        print '+++++ LISTEN UDP ON PORT', port, 'AND RUN STUN DISCOVERY'
        stun.stunExternalIP(close_listener=False, internal_port=port, verbose=False).addBoth(_stuned)

    def _stuned(ip):
        if stun.getUDPClient() is None:
            print 'UDP CLIENT IS NONE - EXIT'
            reactor.stop()
            return

        print '+++++ EXTERNAL UDP ADDRESS IS', stun.getUDPClient().externalAddress
        
        if sys.argv[1] == 'listen':
            print '+++++ START LISTENING'
            return
        
        if sys.argv[1] == 'connect':
            print '+++++ CONNECTING TO REMOTE MACHINE'
            _try2connect()
            return

        lid = misc.getLocalIdentity()
        udp_contact = 'udp://'+stun.getUDPClient().externalAddress[0]+':'+str(stun.getUDPClient().externalAddress[1])
        lid.setProtoContact('udp', udp_contact)
        lid.sign()
        misc.setLocalIdentity(lid)
        misc.saveLocalIdentity()
        
        print '+++++ UPDATE IDENTITY', str(lid.contacts)
        import p2p.identitypropagate
        p2p.identitypropagate.update().addBoth(_id_sent)

    def _try2connect():
        remote_addr = dhnio.ReadTextFile(sys.argv[3]).split(' ')
        remote_addr = (remote_addr[0], int(remote_addr[1]))
        t = int(str(int(time.time()))[-1]) + 1
        data = '0' * t
        stun.getUDPClient().transport.write(data, remote_addr)
        print 'sent %d bytes to %s' % (len(data), str(remote_addr))
        reactor.callLater(1, _try2connect)

    def _id_sent(x):
        print '+++++ ID UPDATED ON THE SERVER'
        if sys.argv[1] == 'send':
            _start_sending()
        elif sys.argv[1] == 'receive':
            _start_receiving()

    def _start_receiving():
        idurl = sys.argv[2]
        if not idurl.startswith('http://'):
            idurl = 'http://'+settings.IdentityServerName()+'/'+idurl+'.xml'
        print '+++++ START RECEIVING FROM', idurl
        _request_remote_id(idurl).addBoth(_receive_from_remote_peer, idurl)
        
    def _receive_from_remote_peer(x, idurl):
        init(stun.getUDPClient())
        contacts.addCorrespondent(idurl)
        reactor.callLater(1, Start)

    def _start_sending():
        idurl = sys.argv[2]
        if not idurl.startswith('http://'):
            idurl = 'http://'+settings.IdentityServerName()+'/'+idurl+'.xml'
        print '+++++ START SENDING TO', idurl
#        if len(sys.argv) == 6:
#            send(sys.argv[5], sys.argv[3], int(sys.argv[4]))
#        elif len(sys.argv) == 4:
        _request_remote_id(idurl).addBoth(_send_to_remote_peer, idurl, sys.argv[3])

    def _request_remote_id(idurl):
        print '+++++ REQUEST ID FROM SERVER', idurl
        return identitycache.immediatelyCaching(idurl)
    
    def _send_to_remote_peer(x, idurl, filename):
        init(stun.getUDPClient())
        reactor.callLater(1, Start)
        ident = identitycache.FromCache(idurl)
        if ident is None:
            print '+++++ REMOTE IDENTITY IS NONE'
            reactor.stop()
        x, udphost, udpport, x = ident.getProtoParts('udp')
        print '+++++ START SENDING TO', udphost, udpport
        reactor.callLater(2, send, filename, udphost, udpport)
    
    
    dhnio.SetDebug(20)
    dhnio.LifeBegins()
    settings.init()
    misc.loadLocalIdentity()
    # contacts.init()
    # contacts.addCorrespondent(idurl)
    identitycache.init()
    port = int(settings.getUDPPort())
    if sys.argv[1] in ['listen', 'connect']:
        port = int(sys.argv[2])
    _go_stun(port)

#------------------------------------------------------------------------------ 

if __name__ == '__main__':
    if len(sys.argv) not in [2, 3, 4, 6] or sys.argv[1] not in ['send', 'receive', 'listen', 'connect']:
        print 'transport_udp.py receive <from username>'
        print 'transport_udp.py receive <from idurl>'
        print 'transport_udp.py send <to username> <filename>'
        print 'transport_udp.py send <to idurl> <filename>'
        print 'transport_udp.py listen <listening port>'
        print 'transport_udp.py connect <listening port> <filename with remote address>'
        sys.exit(0)
        
    main()
    reactor.run()






#!/usr/bin/python
#central_service.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
# This is for services with the central DHN.  Main services are:
#
# 1) register with central service
# 2) list suppliers or customers
# 3) fire a supplier, hire another supplier
# 4) account information and transfers  - done in account.py
# 5) nearnesscheck - meat of function done by nearnesscheck.py
#

import os
import sys
import time
import string

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in central_service.py')

from twisted.internet.defer import Deferred, succeed
from twisted.internet.task import LoopingCall

import lib.misc as misc
import lib.dhnio as dhnio
import lib.settings as settings
import lib.dhnpacket as dhnpacket
import lib.nameurl as nameurl
import lib.packetid as packetid
import lib.commands as commands
import lib.contacts as contacts
import lib.eccmap as eccmap
import lib.transport_control as transport_control
import lib.bandwidth as bandwidth
import lib.diskspace as diskspace
import lib.identitycache as identitycache
import lib.automats as automats

import central_connector
import fire_hire

import identitypropagate
import money
import localtester
import backups
import events
import customerservice


_InitDone = False
_Connector = None
_HelloResponseWaiting = False
_HelloCount = 0
_HelloTask = None
_HelloPID = None
_HelloInterval = 60
_InitControlFunc = None
OnListSuppliersFunc = None
OnListCustomersFunc = None
_CentralStatusDict = {}
_LoopSendBandwidthReportsTask = None

#-------------------------------------------------------------------------------

def init(hello_interaval=60):
    global _InitDone
    if _InitDone:
        return
    dhnio.Dprint(4, 'central_service.init')
    transport_control.AddInboxCallback(inbox)
    bandwidth.init()
    _InitDone = True

def shutdown():
    _LoopSendBandwidthReportsTask.cancel()
    return succeed(1)

#------------------------------------------------------------------------------

def inbox(newpacket, proto, host):
    commandhandled = False

    if newpacket.OwnerID != settings.CentralID():
        return False

    if newpacket.Command == commands.Ack():
        Ack(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.Receipt():
        Receipt(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.ListContacts():
        ListContacts(newpacket)
        commandhandled = True

    if commandhandled:
        dhnio.Dprint(12, "central_service.inbox [%s] from %s://%s handled" % (newpacket.Command, proto, host))

    return commandhandled

def send2central(command, data, doAck=False, PacketID=None):
    MyID = misc.getLocalID()
    RemoteID = settings.CentralID()
    if PacketID is None:
        PacketID = packetid.UniqueID()
    packet = dhnpacket.dhnpacket(
        command,
        MyID,
        MyID,
        PacketID,
        data,
        RemoteID,)
    transport_control.outbox(packet, doAck)
    #del packet
    return PacketID

#------------------------------------------------------------------------------

def HelloRequestSuppliers():
    dhnio.Dprint(4, 'central_service.HelloRequestSuppliers')
    SendRequestSuppliers('', True)


#def LoopSendAlive():
#    dhnio.Dprint(4, 'central_service.LoopSendAlive')
#    SendIdentity()
#    reactor.callLater(settings.DefaultAlivePacketTimeOut(), LoopSendAlive)


#-- SENDING --------------------------------------------------------------------

# Register our identity with central
def SendIdentity(doAck=False):
    dhnio.Dprint(4, "central_service.SendIdentity")
    LocalIdentity = misc.getLocalIdentity()
    data = LocalIdentity.serialize()
    ret = send2central(commands.Identity(), data, doAck, misc.getLocalID())
    return ret

# Say what eccmap we are using for recovery info
# How many suppliers we want (probably same as used by eccmap but just in case)
# Say how much disk we are donating now
def SendSettings(doAck=False, packetID=None):
    if packetID is None:
        packetID = packetid.UniqueID()
    sdict = {}
    sdict['s'] = str(settings.getCentralNumSuppliers())
    # donated = DiskSpace(s=settings.getCentralMegabytesDonated())
    # needed = DiskSpace(s=settings.getCentralMegabytesNeeded())
    # sdict['d'] = str(donated.getValueMb())
    # sdict['n'] = str(needed.getValueMb())
    sdict['d'] = str(diskspace.GetMegaBytesFromString(settings.getCentralMegabytesDonated()))
    sdict['n'] = str(diskspace.GetMegaBytesFromString(settings.getCentralMegabytesNeeded()))
    sdict['e'] = settings.getECC()
    sdict['p'] = str(settings.BasePricePerGBDay())
    sdict['e1'] = str(settings.getEmergencyEmail())
    sdict['e2'] = str(settings.getEmergencyPhone())
    sdict['e3'] = str(settings.getEmergencyFax())
    sdict['e4'] = str(settings.getEmergencyOther()).replace('\n', '<br>')
    sdict['mf'] = settings.getEmergencyFirstMethod()
    sdict['ms'] = settings.getEmergencySecondMethod()
    sdict['ie'] = misc.readExternalIP()
    sdict['il'] = misc.readLocalIP()

    data = dhnio._pack_dict(sdict)
    pid = send2central(commands.Settings(), data, doAck, packetID)
    dhnio.Dprint(4, "central_service.SendSettings PacketID=[%s]" % pid)
    return pid

def SettingsResponse(packet):
    words = packet.Payload.split('\n')
    try:
        status = words[0]
        last_receipt = int(words[1])
    except:
        status = ''
        last_receipt = -1
    dhnio.Dprint(4, "central_service.SettingsResponse: status=[%s] last receipt=[%s]" % (status, str(last_receipt)))

def SendRequestSettings(doAck=False):
    dhnio.Dprint(4, 'central_service.SendRequestSettings')
    return send2central(commands.RequestSettings(), '', doAck)

def SendReplaceSupplier(numORidurl, doAck=False):
    if isinstance(numORidurl, str):
        idurl = numORidurl
    else:
        idurl = contacts.getSupplierID(numORidurl)
    if not idurl:
        dhnio.Dprint(2, "central_service.SendReplaceSupplier ERROR supplier not found")
        return None
    dhnio.Dprint(4, "central_service.SendReplaceSupplier [%s]" % nameurl.GetName(idurl))
    data = 'S\n'+idurl+'\n'+str(contacts.numberForSupplier(idurl))
    ret = send2central(commands.FireContact(), data, doAck)
    events.notify('central_service', 'sent request to dismiss supplier %s' % nameurl.GetName(idurl))
    return ret

def SendChangeSupplier(numORidurl, newidurl, doAck=False):
    if isinstance(numORidurl, str):
        idurl = numORidurl
    else:
        idurl = contacts.getSupplierID(numORidurl)
    if not idurl or not newidurl:
        return None
    dhnio.Dprint(4, "central_service.SendChangeSupplier [%s]->[%s]" % (nameurl.GetName(idurl), nameurl.GetName(newidurl)))
    data = 'N\n'+idurl+'\n'+newidurl
    ret = send2central(commands.FireContact(), data, doAck)

def SendReplaceCustomer(numORidurl, doAck=False):
    if isinstance(numORidurl, str):
        idurl = numORidurl
    else:
        idurl = contacts.getCustomerID(numORidurl)
    if not idurl:
        dhnio.Dprint(2, "central_service.SendReplaceCustomer ERROR customer not found")
        return None
    dhnio.Dprint(4, "central_service.SendReplaceCustomer [%s]" % nameurl.GetName(idurl))
    data = 'C\n'+idurl+'\n'+str(contacts.numberForCustomer(idurl))
    ret = send2central(commands.FireContact(), data, doAck)
    events.notify('central_service', 'sent request to dismiss customer %s' % nameurl.GetName(idurl))
    return ret

def SendRequestSuppliers(data = '', doAck=False):
    dhnio.Dprint(4, "central_service.SendRequestSuppliers")
    return send2central(commands.RequestSuppliers(), data, doAck)

def SendRequestCustomers(data = '', doAck=False):
    dhnio.Dprint(4, "central_service.SendRequestCustomers")
    return send2central(commands.RequestCustomers(), data, doAck)


# Account
# User needs to be able to know his balance. We want to keep it on
#   this machine, not check central each time.  Might have a charge
#   for checking with central and keep date of last checkin so user
#   can not check in too often.
#
# Users who have been with DHN for more than 6 months will be able
#    to transfer some of his prepaid/earned balance to another user.
#    This is like a prepaid cellphone transfering balance to another phone.
#
#    A dhnpacket with a commands.Transfer()
#         The data portion says:
#             Amount: 23.4
#             To: http://foo.bar/baz.xml
#
#    With the dhnpacket signed, we know it is legit.  The ID of the destination is there.
#
#
def SendTransfer(DestinationID, Amount, doAck=False):
    dhnio.Dprint(4, "central_service.SendTransfer  DestinationID=%s  Amount=%s" % (DestinationID, str(Amount)))
    data = DestinationID+'\n'+str(Amount)
    return send2central(commands.Transfer(), data, doAck)


def SendRequestReceipt(missing_receipts, doAck=False):
    dhnio.Dprint(4, 'central_service.SendRequestReceipt ' + str(missing_receipts))
    data = string.join(list(missing_receipts), ' ')
    return send2central(commands.RequestReceipt(), data, doAck)



#--- RECEIVING -----------------------------------------------------------------


def Ack(packet):
    global _HelloResponseWaiting
    dhnio.Dprint(4, "central_service.Ack packetID=[%s]" % packet.PacketID)


def Receipt(request):
    dhnio.Dprint(4, "central_service.Receipt " )
    money.InboxReceipt(request)
    transport_control.SendAck(request)
    missing_receipts = money.SearchMissingReceipts()
    if len(missing_receipts) > 0:
        reactor.callLater(5, SendRequestReceipt, missing_receipts)


#S list of suppliers after RequestSuppliers
#s list of suppliers after RequestSuppliers and have troubles (not found)
#fS list of suppliers after FireContact(supplier)
#fs list of suppliers after FireContact(supplier) and have troubles (not found)
#C list of customers after RequestCustomers
#c list of customers after RequestCustomers and have troubles (not found)
#fC list of customers after FireContact(customers)
#fc list of customers after FireContact(customers) and have troubles (not found)
#bS user was banned with negative balance
legal_codes = ['S','C','s','c','fS','fC','fs','fc','bS']
def ListContacts(request):
    global _CentralStatusDict
    global legal_codes
    global OnListSuppliersFunc
    global OnListCustomersFunc

    data = request.Payload
    words = data.split('\n', 1)
    if len(words) < 2:
        dhnio.Dprint(1, 'central_service.ListContacts ERROR wrong data packet [%s]' % str(request.Payload))
        control('ListContactsError', request)
        return

    code = words[0]
    if not code in legal_codes:
        dhnio.Dprint(1, 'central_service.ListContacts ERROR wrong data in the packet [%s] '  % str(request.Payload))
        control('ListContactsError', request)
        return

    dhnio.Dprint(4, "central_service.ListContacts code=" + str(code) )

    clist, tail = dhnio._unpack_list(words[1])
    current_contacts = contacts.getContactsAndCorrespondents()

    fire_flag = code.startswith('f')
    ban_flag = code.startswith('b')
    contact_type = code.lower()[-1]
    error_flag = code[-1].islower()

    spaceDict = None
    onlineArray = ''
    if tail is not None:
        spaceDict = dhnio._unpack_dict_from_list(tail)
        if spaceDict.has_key('online'):
            onlineArray = spaceDict.pop('online')
    
    #dhnio.Dprint(12, 'central_service.ListContacts fire:[%s] type:[%s] error:[%s]' % (str(fire_flag), str(contact_type), str(error_flag)))

    current_contacts = []
    #---suppliers
    if contact_type == 's':
        current_contacts = contacts.getSupplierIDs()
        contacts.setSupplierIDs(clist)
        eccmap.init()
        contacts.saveSupplierIDs()
        for supid in contacts.getSupplierIDs():
            if supid.strip() == '':
                error_flag = True
        for oldidurl in current_contacts:
            if oldidurl not in clist:
                events.info('central_service', 'supplier %s were disconnected' % nameurl.GetName(oldidurl),)
                misc.writeSupplierData(oldidurl, 'disconnected', time.strftime('%d%m%y %H:%M:%S'))
                dhnio.Dprint(6, 'central_service.ListContacts supplier %s were disconnected' % nameurl.GetName(oldidurl))
        for newidurl in clist:
            if newidurl not in current_contacts:
                transport_control.ClearAliveTime(newidurl)
                misc.writeSupplierData(newidurl, 'connected', time.strftime('%d%m%y %H:%M:%S'))
                events.info('central_service', 'new supplier %s connected' % nameurl.GetName(newidurl), '',)
                dhnio.Dprint(6, 'central_service.ListContacts new supplier %s connected' % nameurl.GetName(newidurl))
        if not fire_flag and current_contacts != clist:
            identitypropagate.suppliers()
            #for newidurl in clist:
            #    identitycache.Remove(newidurl)
            #identitypropagate.RealSendSuppliers()
        backups.SetSupplierList(clist)
        if OnListSuppliersFunc is not None:
            OnListSuppliersFunc()

    #---customers
    elif contact_type == 'c':
        current_contacts = contacts.getCustomerIDs()
        contacts.setCustomerIDs(clist)
        contacts.saveCustomerIDs()
        if spaceDict is not None:
            dhnio._write_dict(settings.CustomersSpaceFile(), spaceDict)
            reactor.callLater(3, localtester.TestUpdateCustomers)
        if not fire_flag and current_contacts != clist:
            identitypropagate.customers()
#            for newidurl in clist:
#                identitycache.Remove(newidurl)
#            identitypropagate.RealSendCustomers()
        if OnListCustomersFunc is not None:
            OnListCustomersFunc()
        for oldidurl in current_contacts:
            if oldidurl not in clist:
                events.info('central_service', 'customer %s were disconnected' % nameurl.GetName(oldidurl),)
                dhnio.Dprint(6, 'central_service.ListContacts customer %s were disconnected' % nameurl.GetName(oldidurl))
        for newidurl in clist:
            if newidurl not in current_contacts:
                transport_control.ClearAliveTime(newidurl)
                events.info('central_service', 'new customer %s connected' % nameurl.GetName(newidurl))
                dhnio.Dprint(6, 'central_service.ListContacts new customer %s connected' % nameurl.GetName(newidurl))

    #---fire_flag
    if fire_flag:
        if contact_type == 's':
            index = -1
            for index in range(len(clist)):
                if clist[index] != current_contacts[index]:
                    break
            if index >= 0:
                # first we wish to update this contacts in identitycache
                # to know his real Identity. to know where to send
                # we just remove it now
                # later his ID will be downloaded automatically
                #identitycache.Remove(clist[index])
                # we want to send our Identity to new supplier
                # and than ask a list of files he have
                # so it should start rebuilding backups immediately
                # right after we got ListFiles from him
                identitypropagate.single(clist[index]) 
                    #lambda ack_packet: customerservice.RequestListFiles(ack_packet.OwnerID))
#                identitypropagate.SendSingleSupplier(clist[index],
#                    lambda ack_packet: customerservice.RequestListFiles(ack_packet.OwnerID))
                # call fire_hire
                #fire_hire.A('hired-new-supplier', (clist, index))

    #---ban_flag
    if ban_flag:
        events.notify('central_service', 'you have negative balance, all your suppliers was removed', '',)
        dhnio.Dprint(2, 'central_service.ListContacts !!! you have negative balance, all your suppliers was removed !!!')

    #---error_flag
    if error_flag:
        #reactor.callLater(settings.DefaultNeedSuppliersPacketTimeOut(), SendRequestSuppliers)
        events.info('central_service', 'could not find available suppliers',
                     'Central server can not find available suppliers for you.\nCheck your central settings.\n',)
        dhnio.Dprint(2, 'central_service.ListContacts !!! could not find available suppliers !!!')

    #identitypropagate.FetchNecesaryContacts()

    for i in range(len(onlineArray)):
        if i < len(clist):
            _CentralStatusDict[clist[i]] = onlineArray[i]  

    #---send ack            
    transport_control.SendAck(request)

    #---automats
    if contact_type == 's':
        central_connector.A('list-suppliers', clist)
        fire_hire.A('list-suppliers', (current_contacts, clist))
    elif contact_type == 'c':
        central_connector.A('list-customers', clist)
        
    #---transport_udp
    if transport_control._TransportUDPEnable:
        import lib.transport_udp as transport_udp
        new_contacts = contacts.getContactsAndCorrespondents()
        transport_udp.ListContactsCallback(current_contacts, new_contacts)



#--- NEARNESS ------------------------------------------------------------------

#  NearnessCheck request coming from central
#  When request comes in we start the check with callback
#  going to NearnessResult which will send results to central.
def NearnessCheck(request):
    dhnio.Dprint(4, "central_service.NearnessCheck")

def NearnessResult():
    dhnio.Dprint(4, "central_service.NearnessResult")

#--- BANDWIDTH -----------------------------------------------------------------

def LoopSendBandwidthReport():
    global _LoopSendBandwidthReportsTask
    if _LoopSendBandwidthReportsTask is not None:
        return
    pid = SendBandwidthReport()
    interval = settings.DefaultBandwidthReportTimeOut()
    if pid is not None:
        transport_control.RegisterInterest(ReceiveBandwidthAck, settings.CentralID(), pid)
    else:
        interval /= 10.0
    _LoopSendBandwidthReportsTask = reactor.callLater(interval, LoopSendBandwidthReport)

def SendBandwidthReport():
    listin, listout = bandwidth.files2send()
    if len(listin) == 0 and len(listout) == 0:
        dhnio.Dprint(4, 'central_service.SendBandwidthReport skip')
        return None
    dhnio.Dprint(4, 'central_service.SendBandwidthReport')
    src = ''
    for filepath in listin:
        filename = os.path.basename(filepath)
        if len(filename) != 6:
            dhnio.Dprint(6, 'central_service.SendBandwidthReport WARNING incorrect filename ' + filepath)
            continue
        s = dhnio.ReadBinaryFile(filepath)
        if not s:
            # dhnio.Dprint(6, 'central_service.SendBandwidthReport WARNING %s is empty' % filepath)
            continue
        src += '[in] %s\n' % filename
        src += s.strip()
        src += '\n[end]\n'
    for filepath in listout:
        filename = os.path.basename(filepath)
        if len(filename) != 6:
            dhnio.Dprint(6, 'central_service.SendBandwidthReport WARNING incorrect filename ' + filepath)
            continue
        s = dhnio.ReadBinaryFile(filepath)
        if not s:
            # dhnio.Dprint(6, 'central_service.SendBandwidthReport WARNING %s is empty' % filepath)
            continue
        src += '[out] %s\n' % filename
        src += s.strip()
        src += '\n[end]\n'
    # dhnio.Dprint(4, '\n' + src)
    if src.strip() == '':
        # dhnio.Dprint(4, 'central_service.SendBandwidthReport WARNING src is empty. skip.')
        return None
    return send2central(commands.BandwidthReport(), src)

def ReceiveBandwidthAck(packet):
    dhnio.Dprint(4, 'central_service.ReceiveBandwidthAck')
    for line in packet.Payload.split('\n'):
        try:
            typ, filename = line.strip().split(' ')
        except:
            continue
        if typ == '[in]':
            filepath = os.path.join(settings.BandwidthInDir(), filename)
        elif typ == '[out]':
            filepath = os.path.join(settings.BandwidthOutDir(), filename)
        else:
            dhnio.Dprint(2, 'central_service.ReceiveBandwidthAck ERROR typ=%s filename=%s' % (typ, filename))
            continue
        if not os.path.isfile(filepath):
            dhnio.Dprint(2, 'central_service.ReceiveBandwidthAck ERROR %s not found' % filepath)
            continue
        if os.path.isfile(filepath+'.sent'):
            dhnio.Dprint(2, 'central_service.ReceiveBandwidthAck WARNING %s already sent' % filepath)
            continue
        try:
            os.rename(filepath, filepath+'.sent')
        except:
            dhnio.Dprint(2, 'central_service.ReceiveBandwidthAck ERROR can not rename %s' % filepath)
            dhnio.DprintException()
        dhnio.Dprint(4, '  %s %s accepted' % (typ, filename))


#-------------------------------------------------------------------------------


def main():
    try:
        from twisted.internet import reactor
    except:
        sys.exit('Error initializing twisted.internet.reactor in central_service.py')

    transport_control.init()
    SendIdentity()
    reactor.run()


if __name__ == '__main__':
    main()







#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
# WebDAV can send and receive files.
# 
# We could have intermediate WebDAV servers, or use WebDAV and certificates instead of ssh.
# Intermediate guys could charge money.
#
# 


#!/usr/bin/env python

# Copyright (c) Twisted Matrix Laboratories.
# See LICENSE for details.

import os
import sys

from twisted.internet.task import LoopingCall 
from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor

import dhnio
import stun

#class PublicUDPServer(DatagramProtocol):
#    def datagramReceived(self, datagram, host):
#        print 'received %d bytes from %s' % (len(datagram), host)
#        if datagram.startswith('test'):
#            self.transport.write('%s:%s' % (host[0], host[1]), host)
#    def send(self, data, address):
#        print 'send %d bytes to %s' % (len(data), address)
#        self.transport.write(data, address)

def main():
    dhnio.SetDebug(18)
    def _dgRcv(datagram, host):
        print 'received %d bytes:[%s] from %s' % (len(datagram), datagram[:128], host)
        if datagram.startswith('test'):
            stun.getUDPClient().transport.write('%s:%s' % (host[0], host[1]), host)
    def _check():
        to = dhnio.ReadTextFile('to').strip()
        if to:
            host, port = to.split(':')
            stun.getUDPClient().transport.write('test', (host, int(port)))
            print 'send "test" to %s' % to
    def _start(x):
        stun.getUDPClient().datagram_received_callback = _dgRcv
        LoopingCall(_check).start(5, False)
    stun.stunExternalIP(close_listener=False, internal_port=int(sys.argv[1])).addBoth(_start)
    
    #t = reactor.listenUDP(int(sys.argv[1]), PublicUDPServer())
        #p = TestClient()
        #l = reactor.listenUDP(int(sys.argv[2]), p)
#    protocol = PublicUDPServer()
#    if len(sys.argv) >= 3:
#        protocol.addr[0] = sys.argv[1]
#        protocol.addr[1] = int(sys.argv[2])
    reactor.run()

if __name__ == '__main__':
    main()



#!/usr/bin/python
#contacts_status.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in contacts_status.py')
    

import lib.dhnio as dhnio
import lib.nameurl as nameurl
import lib.transport_control as transport_control
import lib.contacts as contacts

import lib.automat as automat
#import lib.automats as automats

if transport_control._TransportCSpaceEnable:
    import lib.transport_cspace as transport_cspace


import ratings
#import customerservice


_ContactsByHost = {}
_ContactsStatusDict = {}


#------------------------------------------------------------------------------ 


def init():
    dhnio.Dprint(4, 'contacts_status.init')
    transport_control.AddInboxCallback(Inbox)
    transport_control.AddOutboxCallback(Outbox)
    transport_control.AddOutboxPacketStatusFunc(OutboxStatus)
    if transport_control._TransportCSpaceEnable:
        transport_cspace.SetContactStatusNotifyFunc(CSpaceContactStatus)


def shutdown():
    dhnio.Dprint(4, 'contacts_status.shutdown')
    global _ContactsStatusDict
    for A in _ContactsStatusDict.values():
        automat.clear_object(A.index)
    _ContactsStatusDict.clear()
    

def isOnline(idurl):
    return A(idurl).state == 'CONNECTED'


def isOffline(idurl):
    return A(idurl).state == 'OFFLINE'


def isChecking(idurl):
    return A(idurl).state == 'CHECKING'


def hasOfflineSuppliers():
    for idurl in contacts.getSupplierIDs():
        if isOffline(idurl):
            return True
    return False

#------------------------------------------------------------------------------ 

def check_contacts(contacts_list):
    for idurl in contacts_list:
        A(idurl, 'check') 


def A(idurl, event=None, arg=None):
    global _ContactsStatusDict
    if not _ContactsStatusDict.has_key(idurl):
        _ContactsStatusDict[idurl] = ContactStatus(idurl, 'status_%s' % nameurl.GetName(idurl), 'OFFLINE', 10)
    if event is not None:
        _ContactsStatusDict[idurl].automat(event, arg)
    return _ContactsStatusDict[idurl]
      

class ContactStatus(automat.Automat):
    def __init__(self, idurl, name, state, debug_level):
        self.idurl = idurl
        automat.Automat.__init__(self, name, state, debug_level)
        
    def A(self, event, arg):
        #---CONNECTED---
        if self.state == 'CONNECTED':
            if event in [ 'send-failed', 'cspace-offline' ]: 
                self.state = 'OFFLINE'
            elif event == 'check':
                self.state = 'CHECKING'
            elif event == 'sent-timeout':
                self.doPingContact()
                self.state = 'CHECKING'
        #---OFFLINE---
        elif self.state == 'OFFLINE':
            if event == 'inbox-packet':
                self.state = 'CONNECTED'
            elif event in [ 'outbox-packet', 'check' ]:
                self.state = 'CHECKING'
        #---CHECKING---
        elif self.state == 'CHECKING':
            if event == 'inbox-packet':
                self.state = 'CONNECTED'
            elif event in [ 'send-failed', 'cspace-offline' ]:
                self.state = 'OFFLINE'

    def doPingContact(self):
        reactor.callLater(0, transport_control.PingContact, self.idurl)


def OutboxStatus(workitem, proto, host, status, error, message):
    global _ContactsByHost
    ident = contacts.getContact(workitem.remoteid)
    if ident is not None:
        for contact in ident.getContacts():
            _ContactsByHost[contact] = workitem.remoteid
    if status == 'finished':
        A(workitem.remoteid, 'sent-done', (workitem, proto, host))
    else:
        A(workitem.remoteid, 'send-failed', (workitem, proto, host))


def Inbox(newpacket, proto, host):
    global _ContactsByHost
    ident = contacts.getContact(newpacket.OwnerID)
    if ident is not None:
        for contact in ident.getContacts():
            _ContactsByHost[contact] = newpacket.OwnerID
    A(newpacket.OwnerID, 'inbox-packet', (newpacket, proto, host))
    ratings.remember_connected_time(newpacket.OwnerID)
    

def Outbox(outpacket):
    global _ContactsByHost
    ident = contacts.getContact(outpacket.RemoteID)
    if ident is not None:
        for contact in ident.getContacts():
            _ContactsByHost[contact] = outpacket.RemoteID
    A(outpacket.RemoteID, 'outbox-packet', outpacket)


def CSpaceContactStatus(keyID, status):
    global _ContactsByHost
    idurl = _ContactsByHost.get('cspace://' + keyID, None)
    if idurl is not None:
        A(idurl, 'cspace-'+status, keyID)


def PacketSendingTimeout(remoteID, packetID):
    dhnio.Dprint(6, 'contacts_status.PacketSendingTimeout ' + remoteID)
    A(remoteID, 'sent-timeout', packetID)

    




#!/usr/bin/python
#ratings.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import time
import math


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in ratings.py')

from twisted.internet import task


import lib.dhnio as dhnio
import lib.dhnmath as dhnmath
import lib.misc as misc
import lib.nameurl as nameurl
import lib.settings as settings
import lib.contacts as contacts


import contacts_status


#-------------------------------------------------------------------------------

_LoopCountRatingsTask = None
_IndexMonth = {}
_IndexTotal = {}
_InitDone = False

#-------------------------------------------------------------------------------


def init():
    global _InitDone
    if _InitDone:
        return
    dhnio.Dprint(4, 'ratings.init')
    read_index()
    run()
    _InitDone = True


def shutdown():
    dhnio.Dprint(4, 'ratings.shutdown')
    stop()


def run():
    global _LoopCountRatingsTask
    stop()
    interval = dhnmath.interval_to_next_hour()
    # debug
    #interval = 5
    reactor.callLater(interval, start)
    dhnio.Dprint(6, 'ratings.run will start after %s minutes' % str(interval/60.0))
    
    
def start():
    global _LoopCountRatingsTask
    _LoopCountRatingsTask = task.LoopingCall(rate_all_users)
    _LoopCountRatingsTask.start(settings.DefaultAlivePacketTimeOut())
    dhnio.Dprint(6, 'ratings.start will count ratings every %s minutes' % str(settings.DefaultAlivePacketTimeOut()/60.0))


def stop():
    global _LoopCountRatingsTask
    if _LoopCountRatingsTask:
        if _LoopCountRatingsTask.running:
            _LoopCountRatingsTask.stop()
        del _LoopCountRatingsTask
        _LoopCountRatingsTask = None
        dhnio.Dprint(6, 'ratings.stop task finished')
        
            
def rating_dir(idurl):
    return os.path.join(settings.RatingsDir(), nameurl.UrlFilename(idurl))


def rating_month_file(idurl, monthstr=None):
    if monthstr is None:
        monthstr = time.strftime('%m%y')
    return os.path.join(rating_dir(idurl), monthstr)


def rating_total_file(idurl):
    return os.path.join(rating_dir(idurl), 'total')


def exist_rating_dir(idurl):
    return dhnio._dir_exist(rating_dir(idurl))


def make_rating_dir(idurl):
    dhnio._dir_make(rating_dir(idurl))


def read_month_rating_dict(idurl, monthstr=None):
    if monthstr is None:
        monthstr = time.strftime('%m%y')
    return dhnio._read_dict(rating_month_file(idurl, monthstr))


def write_month_rating_dict(idurl, rating_dict, monthstr=None):
    if monthstr is None:
        monthstr = time.strftime('%m%y')
    return dhnio._write_dict(rating_month_file(idurl, monthstr), rating_dict)


def read_total_rating_dict(idurl):
    return dhnio._read_dict(rating_total_file(idurl))


def write_total_rating_dict(idurl, rating_dict):
    return dhnio._write_dict(rating_total_file(idurl), rating_dict)


def make_blank_rating_dict():
    return {'all':'0', 'alive':'0'}


def increase_rating(idurl, alive_state):
    if not exist_rating_dir(idurl):
        make_rating_dir(idurl)

    month_rating = read_month_rating_dict(idurl)
    if month_rating is None:
        month_rating = make_blank_rating_dict()
    try:
        mallI = int(month_rating['all'])
        maliveI = int(month_rating['alive'])
    except:
        mallI = 0
        maliveI = 0
    mallI += 1
    if alive_state:
        maliveI += 1
    month_rating['all'] = str(mallI)
    month_rating['alive'] = str(maliveI)
    write_month_rating_dict(idurl, month_rating)

    total_rating = read_total_rating_dict(idurl)
    if total_rating is None:
        total_rating = make_blank_rating_dict()
    try:
        tallI = int(total_rating['all'])
        taliveI = int(total_rating['alive'])
    except:
        tallI = 0
        taliveI = 0
    tallI += 1
    if alive_state:
        taliveI += 1
    total_rating['all'] = str(tallI)
    total_rating['alive'] = str(taliveI)
    write_total_rating_dict(idurl, total_rating)
    return mallI, maliveI, tallI, taliveI


def rate_all_users():
    dhnio.Dprint(4, 'ratings.rate_all_users')
    monthStr = time.strftime('%B')
    for idurl in contacts.getContactsAndCorrespondents():
        #isalive = transport_control.ContactIsAlive(idurl)
        isalive = contacts_status.isOnline(idurl)
        mall, malive, tall, talive = increase_rating(idurl, isalive)
        month_percent = 100.0*float(malive)/float(mall)
        total_percent = 100.0*float(talive)/float(tall)
        dhnio.Dprint(4, '[%6.2f%%: %s/%s] in %s and [%6.2f%%: %s/%s] total - %s' % (
            month_percent,
            malive,
            mall,
            monthStr,
            total_percent,
            talive,
            tall,
            nameurl.GetName(idurl),))
    read_index()


def remember_connected_time(idurl):
    if not exist_rating_dir(idurl):
        make_rating_dir(idurl)
    dhnio._write_data(os.path.join(rating_dir(idurl), 'connected'), time.strftime('%d%m%y %H:%M:%S'))
                         

def connected_time(idurl):
    s = dhnio._read_data(os.path.join(rating_dir(idurl), 'connected'))
    if s == '':
        return 0
    try:
        return time.mktime(time.strptime(s, '%d%m%y %H:%M:%S'))
    except:
        return 0


def read_all_monthly_ratings(idurl):
    if not exist_rating_dir(idurl):
        return None
    d = {}
    for monthstr in os.listdir(rating_dir(idurl)):
        if monthstr == 'total':
            continue
        if monthstr == 'last':
            continue
        month_rating = read_month_rating_dict(idurl, monthstr)
        if month_rating is None:
            continue
        d[monthstr] = month_rating
    return d


def read_index(monthstr=None):
    global _IndexMonth
    global _IndexTotal
    #dhnio.Dprint(4, 'ratings.read_index')
    if monthstr is None:
        monthstr = time.strftime('%m%y')
    _IndexMonth.clear()
    _IndexTotal.clear()
    for idurl_filename in os.listdir(settings.RatingsDir()):
        idurl = nameurl.FilenameUrl(idurl_filename)
        if idurl is None:
            continue
        month = read_month_rating_dict(idurl, monthstr)
        total = read_total_rating_dict(idurl)
        _IndexMonth[idurl] = {'all': '0', 'alive': '0'} if month is None else month
        _IndexTotal[idurl] = {'all': '0', 'alive': '0'} if total is None else total
        #dhnio.Dprint(4, '    [%s]: %s, %s' % (nameurl.GetName(idurl), _IndexMonth[idurl], _IndexTotal[idurl]))


def month(idurl):
    global _IndexMonth
    return _IndexMonth.get(idurl, {'all': '0', 'alive': '0'})


def total(idurl):
    global _IndexTotal
    return _IndexTotal.get(idurl, {'all': '0', 'alive': '0'})


def month_percent(idurl):
    try:
        r = month(idurl)
        return round(100.0 * float(r['alive']) / float(r['all']), 2)
    except:
        return 0.0
    

def total_percent(idurl):
    try:
        r = total(idurl)
        return round(100.0 * float(r['alive']) / float(r['all']), 2)
    except:
        return 0.0


#-------------------------------------------------------------------------------


def main():
    pass

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#automats.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in automats.py')
from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet.task import LoopingCall


import lib.dhnio as dhnio
import lib.dhnnet as dhnnet
import lib.misc as misc
import lib.nameurl as nameurl
import lib.packetid as packetid
import lib.settings as settings
import lib.contacts as contacts
import lib.transport_control as transport_control
import lib.diskusage as diskusage
import lib.automat as automat


import initializer
import p2p_connector

import dhninit
import identitypropagate
import install
import webcontrol

#------------------------------------------------------------------------------

_StatesDict = {
    'init at startup':           'beginning',
    'init local':                'local settings initialization',
    'init contacts':             'contacts initialization',
    'init connection':           'initializing connections',
    'init modules':              'starting modules',
    'init install':              'preparing install section',
    'network at startup':        'starting connection',
    'network stun':              'detecting external IP address',
    'network upnp':              'configuring UPnP',
    'network connected':         'internet connection is fine',
    'network disconnected':      'internet connection is not working',
    'network google?':           'is www.google.com available?',
    'p2p at startup':            'initial peer-to-peer state',
    'p2p transports':            'starting network transports',
    'p2p centralservice':        'connecting to central server',
    'p2p propagate':             'propagate my identity',
    'p2p incomming?':            'waiting response from others',
    'p2p connected':             'ready',
    'p2p disconnected':          'starting disconnected',
    'central at startup':        'starting central server connection',
    'central identity':          'sending my identity to central server',
    'central settings':          'sending my settings to central server',
    'central request settings':  'asking my settings from central server',
    'central suppliers':         'requesting suppliers from central server',
    'central connected':         'connected to central server',
    'central disconnected':      'disconnected from central server',
    }

_GlobalState = 'AT_STARTUP'
_GlobalStateNotifyFunc = None

#------------------------------------------------------------------------------

def set_global_state(st):
    global _GlobalState
    global _GlobalStateNotifyFunc
    oldstate = _GlobalState
    _GlobalState = st
    dhnio.Dprint(2, (' ' * 40) + '{%s}->{%s}' % (oldstate, _GlobalState))
    if _GlobalStateNotifyFunc is not None and oldstate != _GlobalState:
        try:
            _GlobalStateNotifyFunc(_GlobalState)
        except:
            dhnio.DprintException()


def get_global_state():
    global _GlobalState
    dhnio.Dprint(6, 'dhninit.get_global_state return [%s]' % _GlobalState)
    return _GlobalState


def get_global_state_label():
    global _GlobalState
    global _StatesDict
    return _StatesDict.get(_GlobalState.replace('_', ' ').lower(), '')


def get_automats_by_index():
    return automat.objects()


def SetGlobalStateNotifyFunc(f):
    global _GlobalStateNotifyFunc
    _GlobalStateNotifyFunc = f
    
def SetSingleStateNotifyFunc(f):
    automat.SetStateChangedCallback(f)

#------------------------------------------------------------------------------











#!/usr/bin/python
#command_line.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import re

try:
    from twisted.internet import reactor
except:
    dhnio.DprintException()
    sys.exit('Error initializing twisted reactor in command_line.py\n')
from twisted.web.client import getPage


import lib.dhnio as dhnio
import lib.misc as misc
import lib.settings as settings
import lib.diskspace as diskspace
import lib.nameurl as nameurl
import lib.contacts as contacts
import lib.dhnpacket as dhnpacket
import lib.packetid as packetid
import lib.commands as commands
import lib.transport_control as transport_control
import lib.schedule as schedule

import backups
import backup_db
import backupshedule
import webcontrol

#------------------------------------------------------------------------------ 

def run(opts, args, overDict, pars):
    print 'Copyright DataHaven.NET LTD. of Anguilla, 2006-2012. All rights reserved.'
    
    if overDict:
        settings.override_dict(overDict)
    settings.init()
    if not opts or opts.debug is None:
        dhnio.SetDebug(0)

    appList = dhnio.find_process([
        'dhnmain.exe',
        'dhnmain.py',
        'dhn.py',
        '/usr/bin/datahaven',
        ])
    running = len(appList) > 0
   
    cmd = ''
    if len(args) > 0:
        cmd = args[0].lower()
    
    #---help---
    if cmd in ['help', 'h']:
        import help
        if len(args) >= 2 and args[1].lower() == 'schedule':
            print help.schedule_format()
        elif len(args) >= 2 and args[1].lower() == 'settings':
            print settings.uconfig().print_all()
        else:
            print help.help()
            print pars.format_option_help()
        return 0
    
    #---backup---
    elif cmd in ['backup', 'backups', 'bk']:
        if not running:
            print 'DataHaven.NET is not running at the moment\n'
            return 0
        return cmd_backups(opts, args, overDict)

    #---restore---
    elif cmd in ['restore', 're']:
        if not running:
            print 'DataHaven.NET is not running at the moment\n'
            return 0
        return cmd_restore(opts, args, overDict)

    #---schedule---
    elif cmd in ['schedule', 'shed', 'sched', 'sh']:
        if not running:
            print 'DataHaven.NET is not running at the moment\n'
            return 0
        return cmd_schedule(opts, args, overDict)

    #---suppliers---
    elif cmd in [ 'suppliers', 'supplier', 'sup', 'supp', 'sp' ]:
        if not running:
            print 'DataHaven.NET is not running at the moment\n'
            return 0
        return cmd_suppliers(opts, args, overDict)

    #---register---
    elif cmd == 'register':
        if running:
            print 'DataHaven.NET already started.\n'
            return 0
        return cmd_register(opts, args, overDict)

    #---recover---
    elif cmd == 'recover':
        if running:
            print 'DataHaven.NET already started.\n'
            return 0
        return cmd_recover(opts, args, overDict)

    #---stats---
    elif cmd in [ 'stats', 'st' ]:
        if not running:
            print 'DataHaven.NET is not running at the moment\n'
            return 0
        return cmd_stats(opts, args, overDict)

    #---version---
    elif cmd in [ 'version', 'v', 'ver' ]:
        revnum = dhnio.ReadTextFile(settings.RevisionNumberFile()).strip()
        repo, location = misc.ReadRepoLocation()
        print 'revision:  ', revnum
        print 'repository:', repo
        return 0

    #---states---
    elif cmd in [ 'states', 'sta', 'automats', 'auto' ]:
        if not running:
            print 'DataHaven.NET is not running at the moment\n'
            return 0
        return cmd_states(opts, args, overDict)

    #---set---
    elif cmd == 'set':
        if len(args) == 1 or args[1].lower() in [ 'help', '?' ]:
            import help
            print help.settings_help()
            return 0
        if not running:
            cmd_set_directly(opts, args, overDict)
            return 0
        return cmd_set_request(opts, args, overDict)
    
    #---memory---
    elif cmd == 'memory':
        if not running:
            print 'DataHaven.NET is not running at the moment\n'
            return 0
        return cmd_memory(opts, args, overDict)
    
#    elif cmd == 'uninstall':
#        return cmd_uninstall(opts, args, overDict)
    
    return 2

#------------------------------------------------------------------------------ 

def run_url_command(address):
    try:
        local_port = int(dhnio.ReadBinaryFile(settings.LocalPortFilename()))
    except:
        print 'can not read file local port number from file %s\n' % settings.LocalPortFilename()
        if reactor.running:
            reactor.stop()
    
    url = 'http://127.0.0.1:'+str(local_port)+'/'+address
    dhnio.Dprint(4, 'command_line.run_url_command url='+url)
    def _eb(x):
        print x.getErrorMessage()
        if reactor.running:
            reactor.stop()
    d = getPage(url)
    d.addErrback(_eb)
    return d


def find_comments(output):
    if output is None or str(output).strip() == '':
        return ['empty output']
    if not isinstance(output, str):
        return [output.getErrorMessage()]
    return re.findall('\<\!--\[begin\] (.+?) \[end\]--\>', output, re.DOTALL)

def print_and_stop(src):
    for s in find_comments(src):
        print unicode(s)
    print
    reactor.stop()
    
#------------------------------------------------------------------------------ 

def print_single_setting_and_stop(path, run_reactor=True):
    if path != '':
        url = webcontrol._PAGE_SETTINGS + '/' + path
        run_url_command(url).addCallback(print_and_stop)
        if run_reactor:
            reactor.run()
        return 0
    return 2

def print_all_settings_and_stop():
    url = webcontrol._PAGE_SETTINGS_LIST
    run_url_command(url).addCallback(print_and_stop)
    reactor.run()
    return 0
    
#------------------------------------------------------------------------------ 

def cmd_backups(opts, args, overDict):
    if len(args) < 2 or args[1] == 'list':
        run_url_command(webcontrol._PAGE_BACKUPS).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif len(args) < 2 or args[1] == 'idlist':
        url = webcontrol._PAGE_BACKUPS+'?byid=show'
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif args[1] == 'start' and len(args) >= 3:
        if not os.path.isdir(os.path.abspath(args[2])):
            print 'folder %s not exist\n' % args[2]
            return 1
        url = webcontrol._PAGE_BACKUPS+'?action=start&backupdir='+misc.pack_url_param(os.path.abspath(args[2]))
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif args[1] == 'add' and len(args) >= 3:
        if not os.path.isdir(os.path.abspath(args[2])):
            print 'folder %s not exist\n' % args[2]
            return 1
        url = webcontrol._PAGE_BACKUPS+'?action=dirselected&opendir='+misc.pack_url_param(os.path.abspath(args[2]))
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif args[1] == 'delete' and len(args) >= 3:
        if args[2] == 'local' and len(args) >= 4:
            url = webcontrol._PAGE_BACKUPS+'/'+args[3]+'?action=deletelocal'
        else:
            if len(args[2]) == 17 and args[2].startswith('F'):
                url = webcontrol._PAGE_BACKUPS + '?action=deleteid&backupid=' + args[2]
            else:
                url = webcontrol._PAGE_BACKUPS + '?action=delete&backupdir=' + misc.pack_url_param(os.path.abspath(args[2]))
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif args[1] == 'update':
        url = webcontrol._PAGE_BACKUPS+'?action=update'
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0
    
    return 2


def cmd_restore(opts, args, overDict):
    if len(args) >= 2:
        url = webcontrol._PAGE_BACKUPS + '?action=restore&backupid=%s' % args[1]
        if len(args) >= 3:
            url += '&destination=%s' % misc.pack_url_param(args[2])
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0
    return 2


def cmd_schedule(opts, args, overDict):
    if len(args) < 2:
        return 2
    if not os.path.isdir(os.path.abspath(args[1])):
        print 'folder %s not exist\n' % args[1]
        return 1
    backupDir = os.path.abspath(args[1])
    if len(args) < 3:
        url = webcontrol._PAGE_BACKUP_SHEDULE + '?backupdir=%s' % (
            misc.pack_url_param(backupDir),)
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0
    shed = schedule.from_compact_string(args[2])
    if shed is None:
        print schedule.format()
        print
        return 0
    url = webcontrol._PAGE_BACKUP_SHEDULE + '?action=save&submit=save&backupdir=%s&type=%s&interval=%s&daytime=%s&details=%s' % (
        misc.pack_url_param(backupDir),
        shed.type, shed.interval, shed.daytime, misc.pack_url_param(shed.details),)
    run_url_command(url).addCallback(print_and_stop)
    reactor.run()
    return 0


def cmd_suppliers(opts, args, overDict):
    def _wait_replace_supplier_and_stop(src, supplier_name, count=0):
        suppliers = []
        for s in find_comments(src):
            if s.count('[online ]') or s.count('[offline]'):
                suppliers.append(s[18:38].strip())
        if supplier_name not in suppliers:
            print '  supplier %s is fired !' % supplier_name
            print_and_stop(src)
            return
        if count >= 20:
            print ' time is out\n'
            reactor.stop()
            return
        else:
            def _check_again(supplier_name, count):
                sys.stdout.write('.')
                run_url_command(webcontrol._PAGE_SUPPLIERS).addCallback(_wait_replace_supplier_and_stop, supplier_name, count)
            reactor.callLater(1, _check_again, supplier_name, count+1)

    if len(args) < 2 or args[1] in [ 'list', 'ls' ]:
        url = webcontrol._PAGE_SUPPLIERS
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif args[1] in [ 'call', 'cl' ]:
        url = webcontrol._PAGE_SUPPLIERS + '?action=call'
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif args[1] in [ 'replace', 'rep', 'rp' ] and len(args) >= 3:
        contacts.init()
        idurl = args[2].strip()
        if not idurl.startswith('http://'):
            try:
                idurl = contacts.getSupplierID(int(idurl))
            except:
                idurl = 'http://'+settings.IdentityServerName()+'/'+idurl+'.xml'
        name = nameurl.GetName(idurl)
        url = webcontrol._PAGE_SUPPLIERS + '?action=replace&idurl=%s' % misc.pack_url_param(idurl)
        run_url_command(url).addCallback(_wait_replace_supplier_and_stop, name, 0)
        reactor.run()
        return 0
    
    elif args[1] in [ 'change', 'ch' ] and len(args) >= 4:
        contacts.init()
        idurl = args[2].strip()
        if not idurl.startswith('http://'):
            try:
                idurl = contacts.getSupplierID(int(idurl))
            except:
                idurl = 'http://'+settings.IdentityServerName()+'/'+idurl+'.xml'
        newidurl = args[3].strip()
        if not newidurl.startswith('http://'):
            newidurl = 'http://'+settings.IdentityServerName()+'/'+newidurl+'.xml'
        name = nameurl.GetName(idurl)
        newname = nameurl.GetName(newidurl)
        url = webcontrol._PAGE_SUPPLIERS + '?action=change&idurl=%s&newidurl=%s' % (misc.pack_url_param(idurl), misc.pack_url_param(newidurl))
        run_url_command(url).addCallback(_wait_replace_supplier_and_stop, name, 0)
        reactor.run()
        return 0
    
    return 2


def cmd_register(opts, args, overDict):
    if len(args) < 2:
        return 2
    import lib.automats as automats
    import initializer
    import shutdowner
    initializer.A('run-cmd-line-register', args[1])
    reactor.run()
    #initializer.A('reactor-stopped', use_reactor = False)
    shutdowner.A('reactor-stopped')
    automats.get_automats_by_index().clear()
    print
    return 0


def cmd_recover(opts, args, overDict):
    if len(args) < 2:
        return 2
    src = dhnio.ReadBinaryFile(args[1])
    if len(src) > 1024*10:
        print 'file is too big for private key'
        return 0
    try:
        lines = src.split('\n')
        idurl = lines[0]
        txt = '\n'.join(lines[1:])
        if idurl != nameurl.FilenameUrl(nameurl.UrlFilename(idurl)):
            idurl = ''
            txt = src
    except:
        #dhnio.DprintException()
        idurl = ''
        txt = src
    if idurl == '' and len(args) >= 3:
        idurl = args[2]
        if not idurl.startswith('http://'):
            idurl = 'http://'+settings.IdentityServerName()+'/'+idurl+'.xml'
    if idurl == '':
        print 'DataHaven.NET need to know your username to recover your account\n'
        return 2
    import lib.automats as automats
    import initializer
    import shutdowner
    initializer.A('run-cmd-line-recover', { 'idurl': idurl, 'keysrc': txt })
    reactor.run()
    #initializer.A('reactor-stopped', use_reactor = False)
    shutdowner.A('reactor-stopped')
    automats.get_automats_by_index().clear()
    print
    return 0


def cmd_stats(opts, args, overDict):
    if len(args) == 2:
        url = '%s/%s' % (webcontrol._PAGE_BACKUPS, args[1])
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif len(args) >= 3 and args[1] == 'local': 
        url = '%s/%s/%s' % (webcontrol._PAGE_BACKUPS, args[2], webcontrol._PAGE_BACKUP_LOCAL_FILES)
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0

    elif len(args) >= 3 and args[1] == 'remote': 
        url = '%s/%s/%s' % (webcontrol._PAGE_BACKUPS, args[2], webcontrol._PAGE_BACKUP_REMOTE_FILES)
        run_url_command(url).addCallback(print_and_stop)
        reactor.run()
        return 0
    
    return 2

def cmd_states(opts, args, overDict):
    url = '%s' % (webcontrol._PAGE_AUTOMATS)
    run_url_command(url).addCallback(print_and_stop)
    reactor.run()
    return 0
    
def option_name_to_path(name, default=''):
    path = default
    if name in [ 'donated', 'shared', 'given', ]:
        path = 'central-settings.shared-megabytes'
    elif name in [ 'needed', ]:
        path = 'central-settings.needed-megabytes'
    elif name in [ 'suppliers', ]:
        path = 'central-settings.desired-suppliers'
    elif name in [ 'debug' ]:
        path = 'logs.debug-level'
    return path

def cmd_set_directly(opts, args, overDict):
    def print_all_settings():
        for path in settings.uconfig().default_order:
            if path.strip() == '':
                continue
            value = settings.uconfig().data.get(path, '').replace('\n', ' ')
            label = settings.uconfig().labels.get(path, '')
            info = settings.uconfig().infos.get(path, '')
            print '  %s    %s    %s' % (label.ljust(30), value.ljust(20)[:20], path)
        return 0
    name = args[1].lower()
    if name in [ 'list' ]:
        return print_all_settings() 
    path = '' if len(args) < 2 else args[1]
    path = option_name_to_path(name, path)
    if path != '':
        old_is = settings.uconfig().get(path)
        if len(args) > 2:
            value = ' '.join(args[2:])
            settings.uconfig().set(path, unicode(value))
            settings.uconfig().update()
        print '  XML path: %s' % path
        print '  label:    %s' % settings.uconfig().get(path, 'label')
        print '  info:     %s' % settings.uconfig().get(path, 'info')
        print '  value:    %s' % settings.uconfig().get(path)
        if len(args) > 2:
            print '  modified: [%s]->[%s]' % (old_is, value)
        return 0
    
def cmd_set_request(opts, args, overDict):
    name = args[1].lower()
    if name in [ 'list' ]:
        return print_all_settings_and_stop() 
    path = '' if len(args) < 2 else args[1]
    path = option_name_to_path(name, path)    
    if len(args) == 2:
        return print_single_setting_and_stop(path)
    action = 'action='
    leafs = path.split('.')
    name = leafs[-1]
    webcontrol.init_settings_tree()
    cls = webcontrol._SettingsTreeNodesDict.get(name, None)
    input = ' '.join(args[2:])
    #print cls, name, path, input
    if cls is None:
        return 2
    if cls in [ webcontrol.SettingsTreeTextNode,
                webcontrol.SettingsTreeUStringNode,
                webcontrol.SettingsTreePasswordNode,
                webcontrol.SettingsTreeNumericNonZeroPositiveNode,
                webcontrol.SettingsTreeNumericPositiveNode,] :
        action = 'text=' + misc.pack_url_param(input)
    elif cls in [ webcontrol.SettingsTreeDiskSpaceNode, ]:
        number = misc.DigitsOnly(input, '.')
        suffix = input.lstrip('0123456789.-').strip()
        action = 'number=%s&suffix=%s' % (number, suffix)
    elif cls in [ webcontrol.SettingsTreeComboboxNode, ]:
        number = misc.DigitsOnly(input)
        action = 'choice=%s' % number
    elif cls in [ webcontrol.SettingsTreeYesNoNode, ]:
        trueORfalse = 'True' if input.lower().strip() == 'true' else 'False'
        action = 'choice=%s' % trueORfalse
    url = webcontrol._PAGE_SETTINGS + '/' + path + '?' + action
    run_url_command(url).addCallback(lambda src: print_single_setting_and_stop(path, False)) #.addCallback(print_and_stop)
    reactor.run()
    return 0


def cmd_memory(opts, args, overDict):
    url = webcontrol._PAGE_MEMORY
    run_url_command(url).addCallback(print_and_stop)
    reactor.run()
    return 0
    

def cmd_uninstall(opts, args, overDict):
    if not dhnio.Windows():
        print 'This command can be used only under OS Windows.'
        return 0
    if not dhnio.isFrozen():
        print 'You are running DataHaven.NET from sources, uninstall command is available only for binary version.'
        return 0
    def do_uninstall():
        dhnio.Dprint(0, 'command_line.do_uninstall')
        batfilename = misc.MakeBatFileToUninstall()
        misc.UpdateRegistryUninstall(True)
        misc.RunBatFile(batfilename, 'c:/out2.txt')
    def kill():
        dhnio.Dprint(0, 'kill')
        total_count = 0
        found = False
        while True:
            appList = dhnio.find_process([
                'dhnmain.exe',
                'dhnmain.py',
                'dhn.py',
                '/usr/bin/datahaven',
                'dhnview.exe',
                'dhnview.py',
                'dhnbackup.exe',
                'dhnbackup.py',
                'dhntester.exe',
                'dhntester.py',
                'dhnstarter.exe',
                ])
            if len(appList) > 0:
                found = True
            for pid in appList:
                dhnio.Dprint(0, 'trying to stop pid %d' % pid)
                dhnio.kill_process(pid)
            if len(appList) == 0:
                if found:
                    dhnio.Dprint(0, 'DataHaven.NET stopped\n')
                else:
                    dhnio.Dprint(0, 'DataHaven.NET was not started\n')
                return 0
            total_count += 1
            if total_count > 10:
                dhnio.Dprint(0, 'some DataHaven.NET process found, but can not stop it\n')
                return 1
            time.sleep(1)            
    def wait_than_kill(x):
        dhnio.Dprint(0, 'wait_than_kill')
        total_count = 0
        #while True:
        def _try():
            dhnio.Dprint(0, '_try')
            appList = dhnio.find_process([
                'dhnmain.exe',
                'dhnview.exe',
                'dhnbackup.exe',
                'dhntester.exe',
                'dhnstarter.exe',
                ])
            dhnio.Dprint(0, 'appList:' + str(appList))
            if len(appList) == 0:
                dhnio.Dprint(0, 'finished')
                reactor.stop()
                do_uninstall()
                return 0
            total_count += 1
            dhnio.Dprint(0, '%d' % total_count)
            if total_count > 10:
                dhnio.Dprint(0, 'not responding')
                ret = kill()
                reactor.stop()
                if ret == 0:
                    do_uninstall()
                return ret
            reactor.callLater(1, _try)
        _try()
#            time.sleep(1)
    appList = dhnio.find_process([
        'dhnmain.exe',
        'dhnview.exe',
        'dhnbackup.exe',
        'dhntester.exe',
        'dhnstarter.exe',
        ])
    if len(appList) == 0:
        dhnio.Dprint(0, 'uninstalling DataHaven.NET ...   ')
        do_uninstall()
        return 0
    dhnio.Dprint(0, 'found DataHaven.NET processes ...   ')
    try:
        url = webcontrol._PAGE_ROOT+'?action=exit'
        run_url_command(url).addCallback(wait_than_kill)
        #reactor.addSystemEventTrigger('before', 'shutdown', do_uninstall)
        reactor.run()
        return 0
    except:
        dhnio.DprintException()
        ret = kill()
        if ret == 0:
            do_uninstall()
        return ret
    


#def cmd_msg(opts, args, overDict):
#    if len(dhnio.find_process([
#            'dhnmain.exe', 
#            'dhnmain.py', 
#            'dhn.py', 
#            '/usr/bin/datahaven' ])) > 0:
#        dhnio.Dprint(2, 'dhnmain.main DHN already running. EXIT.')
#        return 1
#    idurl = args[1]
#    text = ' '.join(args[2:])
#    dhnio.Dprint(2, 'dhnmain.cmd_msg to: ' + idurl)
#    dhnio.Dprint(2, 'dhnmain.cmd_msg text:')
#    dhnio.Dprint(0, text)
#    import message
#    try:
#        from twisted.internet import reactor
#    except:
#        dhnio.DprintException()
#        sys.exit('Error initializing twisted reactor in dhnmain.py\n')
#
#    def _send():
#        dhnio.Dprint(2, 'dhnmain.cmd_msg._send')
#        msgbody = message.MakeMessage(idurl, 'dhnmain short message', text)
#        message.SendMessage(idurl, msgbody)
#        message.SaveMessage(msgbody)
#
#    dhnio.Dprint(2, 'dhnmain.cmd_msg want to import dhninit')
#    import dhninit
#    dhninit.initDoneFunc = _send
#
#    UI = ''
#    dhnio.Dprint(2, 'dhnmain.cmd_msg want to call dhninit.run("%s")' % UI)
#    dhninit.run(UI, opts, args, overDict)
#    dhnio.Dprint(2, 'dhnmain.cmd_msg will call reactor.run()')
#    reactor.run()
#    return 0
#
#
#def cmd_tc(opts, args, overDict):
#    import lib.dhnio as dhnio
#    import lib.settings as settings
#    import lib.transport_control as transport_control
#    try:
#        from twisted.internet import reactor
#    except:
#        dhnio.DprintException()
#        sys.exit('Error initializing twisted reactor in dhnmain.py\n')
#    if overDict:
#        settings.override_dict(overDict)
#    settings.init()
#    if args[1] == 'init':
#        dhnio.Dprint(2, 'dhnmain.cmd_tc will call transport_control.init()')
#        transport_control.init()
#        reactor.run()
#    elif args[1] == 'ping':
#        from twisted.internet import task
#        import lib.misc as misc
#        import lib.dhnpacket as dhnpacket
#        import lib.packetid as packetid
#        import lib.commands as commands
#        import lib.contacts as contacts
#        idurl = args[2]
#        if not idurl.startswith('http://'):
#            idurl = misc.username2idurl(idurl)
#        interval = float(args[3])
#        dhnio.Dprint(2, 'dhnmain.cmd_tc want to ping %s' % idurl)
#        def _loop():
#            dhnio.Dprint(6, 'dhnmain.cmd_tc._loop')
#            LocalIdentity = misc.getLocalIdentity()
#            data = LocalIdentity.serialize()
#            packet = dhnpacket.dhnpacket(
#                commands.Identity(),
#                misc.getLocalID(),
#                misc.getLocalID(),
#                packetid.UniqueID(),
#                data,
#                idurl,)
#            transport_control.outbox(packet, True)
#            del packet
#        def _init_cb():
#            dhnio.Dprint(4, 'dhnmain.cmd_tc._init_cb')
#            t = task.LoopingCall(_loop)
#            t.start(interval)
#        contacts.init()
#        transport_control.init(_init_cb)
#        reactor.run()
#    else:
#        dhnio.Dprint(2, 'dhnmain.cmd_tc ERROR wrong command line arguments')
#        return 2
#    return 0
#
#
#def cmd_tcp(opts, args, overDict):
#    import lib.dhnio as dhnio
#    import lib.transport_tcp as transport_tcp
#    try:
#        from twisted.internet import reactor
#    except:
#        dhnio.DprintException()
#        sys.exit('Error initializing twisted reactor in dhnmain.py\n')
#    if args[1] == 'receive':
#        port = args[2]
#        dhnio.Dprint(2, 'dhnmain.cmd_tcp receive on port ' + port)
#        transport_tcp.receive(port)
#        reactor.run()
#    elif args[1] == 'send':
#        host = args[2]
#        port = args[3]
#        filename = args[4]
#        dhnio.Dprint(2, 'dhnmain.cmd_tcp send to %s:%s filename=[%s] ' % (host, port, filename))
#        def _send(x):
#            dhnio.Dprint(2, 'dhnmain.cmd_tcp result=[%s]' % str(x))
#            reactor.stop()
#        r = transport_tcp.send(filename, host, port)
#        r.addBoth(_send)
#        reactor.run()
#    else:
#        dhnio.Dprint(2, 'dhnmain.cmd_tcp ERROR wrong command line arguments')
#        return 2
#    return 0
#
#
#def cmd_q2q(opts, args, overDict):
#    import lib.dhnio as dhnio
#    import lib.settings as settings
#    import lib.transport_q2q as transport_q2q
#    try:
#        from twisted.internet import reactor
#    except:
#        dhnio.DprintException()
#        sys.exit('Error initializing twisted reactor in dhnmain.py\n')
#    if overDict:
#        settings.override_dict(overDict)
#    settings.init()
#    if args[1] == 'register':
#        userhost = args[2]
#        password = args[3]
#        dhnio.Dprint(2, 'dhnmain.cmd_q2q register new user ' + userhost)
#        def init_callback(x):
#            dhnio.Dprint(4, 'dhnmain.cmd_q2q.init_callback: ' + str(x))
#            transport_q2q.register(userhost, password).addBoth(lambda x: reactor.stop())
#        transport_q2q.init(False).addBoth(init_callback)
#        reactor.run()
#    elif args[1] == 'receive':
#        dhnio.Dprint(2, 'dhnmain.cmd_q2q receive')
#        def init_callback(x):
#            dhnio.Dprint(4, 'dhnmain.cmd_q2q.init_callback: ' + str(x))
#            transport_q2q.receive()
#        transport_q2q.init(False).addBoth(init_callback)
#        reactor.run()
#    elif args[1] == 'send':
#        to_userhost = args[2]
#        filename = args[3]
#        timeout = None
#        if len(args) > 4:
#            timeout = int(args[4])
#        dhnio.Dprint(2, 'dhnmain.cmd_q2q send to %s file %s' % (to_userhost, filename))
#        def init_callback(x):
#            dhnio.Dprint(4, 'dhnmain.cmd_q2q.init_callback: ' + str(x))
#            if timeout:
#                t = LoopingCall(transport_q2q.send, to_userhost, filename)
#            else:
#                transport_q2q.send(to_userhost, filename).addBoth(lambda x: reactor.stop())
#        transport_q2q.init(False).addBoth(init_callback)
#        reactor.run()
#    elif args[1] == 'clear':
#        transport_q2q.clear_local_db(settings.Q2QDir())
#    else:
#        dhnio.Dprint(2, 'dhnmain.cmd_tcp ERROR wrong command line arguments')
#        return 2
#    return 0




#!/usr/bin/env python
#identity_registrator.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import time

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in identity_registrator.py')
from twisted.internet.defer import Deferred, DeferredList, maybeDeferred
from twisted.internet.task import LoopingCall


from lib.automat import Automat
import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.settings as settings
import lib.identitycache as identitycache
import lib.stun as stun
import lib.identity as identity
import lib.dhncrypto as dhncrypto
import lib.tmpfile as tmpfile
import lib.dhnnet as dhnnet
import lib.transport_control as transport_control
import lib.transport_tcp as transport_tcp


import installer
import p2p_connector
import lib.automats as automats

import dhninit
import identitypropagate
import webcontrol

_IdentityRegistrator = None
_NewIdentity = None

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _IdentityRegistrator
    if _IdentityRegistrator is None:
        _IdentityRegistrator = IdentityRegistrator('identity_registrator', 'READY', 4)
    if event is not None:
        _IdentityRegistrator.automat(event, arg)
    return _IdentityRegistrator


class IdentityRegistrator(Automat):
    timers = {'timer-30sec': (30, ['REQUEST_MY_ID']),}
    MESSAGES = {
        'MSG_01': ['checking account name'],
        'MSG_02': ['user %(login)s already exist', 'red'],
        'MSG_03': ['checking network configuration'],
        'MSG_04': ['local IP is %(localip)s'],
        'MSG_05': ['network connection failed', 'red'],
        'MSG_06': ['external IP is %(externalip)s'],
        'MSG_07': ['network connection error', 'red'],
        'MSG_08': ['sending my identity to the identity server'],
        'MSG_09': ['connection error while sending my identity', 'red'],
        'MSG_10': ['verifying my identity on the server'],
        'MSG_11': ['time out connection to the identity server', 'red'],
        'MSG_12': ['verifying my identity'],
        'MSG_13': ['identity verification failed', 'red'],
        'MSG_14': ['new user %(login)s registered successfully!', 'green'], 
        'MSG_15': ['connecting to the identity server'], 
        'MSG_16': ['connection to the identity server was failed', 'red'], }

    def msg(self, arg): 
        msg = self.MESSAGES.get(arg, ['', 'black'])
        text = msg[0] % {
            'login': dhnio.ReadTextFile(settings.UserNameFilename()),
            'externalip': dhnio.ReadTextFile(settings.ExternalIPFilename()),
            'localip': dhnio.ReadTextFile(settings.LocalIPFilename()),}
        color = 'black'
        if len(msg) == 2:
            color = msg[1]
        return text, color

    def state_changed(self, oldstate, newstate):
        automats.set_global_state('ID_REGISTER ' + newstate)
        installer.A('identity_registrator.state', newstate)

    def A(self, event, arg):
        #---READY---
        if self.state == 'READY':
            if event == 'start':
                installer.A('print', self.msg('MSG_15'))
                self.doSaveMyName(arg)
                self.doPingIdentityServer()
                self.state = 'ID_SERVER'
        #---ID_SERVER---
        elif self.state == 'ID_SERVER':
            if event == 'id-server-response':
                installer.A('print', self.msg('MSG_01'))
                self.doRequestMyIdentity()
                self.state = 'USER_NAME'
            elif event == 'id-server-failed':
                installer.A('print', self.msg('MSG_16'))
                self.state = 'READY'
        #---USER_NAME---
        elif self.state == 'USER_NAME':
            if event == 'my-id-exist':
                installer.A('print', self.msg('MSG_02'))
                self.state = 'READY'
            elif event == 'my-id-not-exist':
                installer.A('print', self.msg('MSG_03'))
                self.doDetectLocalIP()
                self.state = 'LOCAL_IP'
        #---LOCAL_IP---
        elif self.state == 'LOCAL_IP':
            if event == 'local-ip-detected':
                installer.A('print', self.msg('MSG_04'))
                self.doStunExternalIP()
                self.state = 'EXTERNAL_IP'
        #---EXTERNAL_IP---
        elif self.state == 'EXTERNAL_IP':
            if event == 'stun-success':
                installer.A('print', self.msg('MSG_06'))
                self.doRequestCentralIdentity()
                self.state = 'CENTRAL_ID'
            elif event == 'stun-failed':
                installer.A('print', self.msg('MSG_05'))
                self.state = 'READY' 
        #---CENTRAL_ID---
        elif self.state == 'CENTRAL_ID':
            if event == 'central-id-success':
                installer.A('print', self.msg('MSG_08'))
                self.doCreateMyIdentity()
                self.doSendMyIdentity()
                self.state = 'SEND_MY_ID'
            elif event == 'central-id-failed':
                installer.A('print', self.msg('MSG_07'))
                self.state = 'READY' 
        #---SEND_MY_ID---
        elif self.state == 'SEND_MY_ID':
            if event == 'my-id-sent':
                installer.A('print', self.msg('MSG_10'))
                self.doRequestMyIdentity()
                self.state = 'REQUEST_MY_ID'
            elif event == 'my-id-failed':
                installer.A('print', self.msg('MSG_09'))
                self.state = 'READY' 
        #---REQUEST_MY_ID---
        elif self.state == 'REQUEST_MY_ID':
            if event == 'my-id-exist' and self.isMyIdentityValid(arg):
                self.doSaveMyIdentity()
                installer.A('print', self.msg('MSG_14'))
                self.state = 'REGISTERED'
            elif event == 'my-id-exist' and not self.isMyIdentityValid(arg):
                installer.A('print', self.msg('MSG_13'))
                self.state = 'READY' 
            elif event == 'my-id-not-exist':
                self.doWait5sec().addCallback(
                    lambda x: self.doRequestMyIdentity())
            elif event == 'timer-30sec':
                installer.A('print', self.msg('MSG_11'))
        #---REGISTERED---
        elif self.state == 'REGISTERED':
            pass
        
    def isMyIdentityValid(self, arg):
        global _NewIdentity
        return _NewIdentity.serialize() == arg

    def doSaveMyName(self, arg):
        login = arg
        dhnio.WriteFile(settings.UserNameFilename(), login)
        webcontrol.installing_process_str = ''

    def doPingIdentityServer(self):
        server_url = nameurl.UrlMake('http', settings.IdentityServerName())
        dhnnet.getPageTwisted(server_url).addCallbacks(
            lambda src: self.automat('id-server-response', src),
            lambda err: self.automat('id-server-failed', err))

    def doRequestMyIdentity(self):
        login = dhnio.ReadTextFile(settings.UserNameFilename())
        idurl = nameurl.UrlMake('http', settings.IdentityServerName(), '', login+'.xml')
        dhnio.Dprint(4, 'identity_registrator.doRequestMyIdentity login=%s, idurl=%s' % (login, idurl))
        dhnnet.getPageTwisted(idurl).addCallbacks(
            lambda src: self.automat('my-id-exist', src),
            lambda err: self.automat('my-id-not-exist', err))
        
    def doDetectLocalIP(self):
        localip = dhnnet.getLocalIp()
        dhnio.WriteFile(settings.LocalIPFilename(), localip)
        dhnio.Dprint(4, 'identity_registrator.doDetectLocalIP [%s]' % localip)
        self.automat('local-ip-detected')
        
    def doStunExternalIP(self):
        dhnio.Dprint(4, 'identity_registrator.doStunExternalIP')
        def save(ip):
            dhnio.Dprint(4, 'identity_registrator.doStunExternalIP.save [%s]' % ip)
            dhnio.WriteFile(settings.ExternalIPFilename(), ip)
            self.automat('stun-success', ip)
        stun.stunExternalIP(close_listener=False, internal_port=settings.getUDPPort()).addCallbacks(
            save, lambda x: self.automat('stun-failed'))

    def doRequestCentralIdentity(self):
        identitycache.immediatelyCaching(settings.CentralID()).addCallbacks(
            lambda x: self.automat('central-id-success'),
            lambda x: self.automat('central-id-failed'))
        
    def doCreateMyIdentity(self):
        CreateNewIdentity()
        
    def doSendMyIdentity(self):
        dl = SendNewIdentity()
        dl.addCallback(lambda x: self.automat('my-id-sent'))
        dl.addErrback(lambda x: self.automat('my-id-failed'))
        
    def doSaveMyIdentity(self):
        global _NewIdentity
        misc.setLocalIdentity(_NewIdentity)
        misc.saveLocalIdentity()
        
    def doWait5sec(self):
        d = Deferred()
        reactor.callLater(5, d.callback, 0)
        return d
        
def CreateNewIdentity():
    global _NewIdentity
    
    misc.loadLocalIdentity()
    if misc.isLocalIdentityReady():
        if misc.getLocalIdentity().Valid():
            _NewIdentity = misc.getLocalIdentity()
            return
        else:
            dhnio.Dprint(2, 'identity_registrator.CreateNewIdentity ERROR local identity is not VALID!!!')

    login = dhnio.ReadTextFile(settings.UserNameFilename())
    externalIP = dhnio.ReadTextFile(settings.ExternalIPFilename())
    localIP = dhnio.ReadTextFile(settings.LocalIPFilename())

    dhnio.Dprint(4, 'identity_registrator.CreateNewIdentity %s %s ' % (login, externalIP))
    
    idurl = 'http://'+settings.DefaultIdentityServer()+'/'+login.lower()+'.xml'
    ident = identity.identity( )
    ident.sources.append(idurl)

    cdict = {}
    if settings.enableTCP():
        cdict['tcp'] = 'tcp://'+externalIP+':'+settings.getTCPPort()
    if settings.enableCSpace() and transport_control._TransportCSpaceEnable:
        cdict['cspace'] = 'cspace://'
        if settings.getCSpaceKeyID() != '':
            cdict['cspace'] += settings.getCSpaceKeyID()
    if settings.enableUDP() and transport_control._TransportUDPEnable:
        if stun.getUDPClient() is not None:
            if stun.getUDPClient().externalAddress is not None: # _altStunAddress
                cdict['udp'] = 'udp://'+stun.getUDPClient().externalAddress[0]+':'+str(stun.getUDPClient().externalAddress[1])
        
    for c in misc.validTransports:
        if cdict.has_key(c):
            ident.contacts.append(cdict[c])

    ident.publickey = dhncrypto.MyPublicKey()
    ident.date = time.ctime() #time.strftime('%b %d, %Y')

    revnum = dhnio.ReadTextFile(settings.RevisionNumberFile()).strip()
    repo, location = misc.ReadRepoLocation()
    ident.version = (revnum.strip() + ' ' + repo.strip() + ' ' + dhnio.osinfo().strip()).strip()

    ident.sign()
    
    dhnio.WriteFile(settings.LocalIdentityFilename()+'.new', ident.serialize())
    
    _NewIdentity = ident
    

def SendNewIdentity():
    global _NewIdentity
    dhnio.Dprint(4, 'identity_registrator.SendNewIdentity ')

    sendfile, sendfilename = tmpfile.make("propagate")
    os.close(sendfile)
    src = _NewIdentity.serialize()
    dhnio.WriteFile(sendfilename, src)

    dlist = []
    for idurl in _NewIdentity.sources:            
        # sources for out identity are servers we need to send to
        protocol, host, port, filename = nameurl.UrlParse(idurl)
        port = settings.IdentityServerPort()
        dlist.append(transport_tcp.send(sendfilename, host, port, False))

    dl = DeferredList(dlist)
    return dl
                
        
        





#!/usr/bin/python
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
# If all commands are repeatable, then sequence numbers are not so critical,
#   though we would want date so time for replay trouble was limited.
#  If backups are write, read, delete (and never write again), then replay
#   is not an issue here and we use PacketID that identifies data.
#  So we want things like "replace supplier Vincecate"  not "replace supplier 5" where seeing command an extra time would not hurt.


#  These are the valid values for the command field of a dhnpacket
#        Data/Ack/                 (if customer sends you data you store it) (could be parity packet too)
#        Retrieve/Data|Fail        (data packets returned exactly as is with our signature)
#        ListFiles/Files   (ask supplier to list backup IDs he knows about for us)
#    To DHN
#        ListContacts/Contacts
#    From DHN
#        NearnessCheck/Nearness

def IsCommand(s):                 # check to see if s is a valid command
    return IsP2PCommand(s) or IsCentralCommand(s)

P2PCommandAcks={}
CentralCommandAcks={}

def init():
    initP2PCommands()
    initCentralCommands()

# These are command strings sent P2P in "command" field of dhnpackets only
def initP2PCommands():
    global P2PCommandAcks
    P2PCommandAcks[Ack()]=NoAck()                         # No Ack for Ack
    P2PCommandAcks[Fail()]=NoAck()                        # No Ack for Fail

    P2PCommandAcks[Data()]=Ack()                          # Ack with Ack unless it is our data coming back (would be only after Retrieve)
    P2PCommandAcks[Retrieve()]=Data()                     # Ack with Data

    P2PCommandAcks[ListFiles()]=Files()                   # Ack ListFiles with Files
    P2PCommandAcks[Files()]=NoAck()

    P2PCommandAcks[ListContacts()]=Contacts()             # Ack with Contacts
    P2PCommandAcks[Contacts()]=NoAck()

    P2PCommandAcks[NearnessCheck()]=Nearness()            # Ack with Nearness
    P2PCommandAcks[Nearness()]=NoAck()

    P2PCommandAcks[RequestIdentity()]=Identity()          # Ack with Identity (always an interested party waiting)
    P2PCommandAcks[Identity()]=Ack()                      # If identity comes in and no interested party then transport sends an Ack

    P2PCommandAcks[DeleteFile()]=Ack()                    # Ack with Ack (maybe should be Files)
    P2PCommandAcks[DeleteBackup()]=Ack()                  # Ack with Ack (maybe should be Files)
    P2PCommandAcks[Message()]=Ack()                       # Ack with Ack
    P2PCommandAcks[Receipt()]=Ack()                       # Ack with Ack
    
    P2PCommandAcks[Correspondent()]=Correspondent()

def IsP2PCommand(s):                      # check to see if s is a valid command
    global P2PCommandAcks
    result = s in P2PCommandAcks
    # print "Commands.IsP2PCommand given %s  is returning %s" % (s, result)
    return result

def AckForCommand(s):                     # Return the command we expect back after this type of command
    global P2PCommandAcks
    global CentralCommandAcks
    if s in P2PCommandAcks:
        return P2PCommandAcks[s]
    if s in CentralCommandAcks:
        return CentralCommandAcks[s]
    raise Exception("Commands.AckForCommand asked for Ack for non command " + s)

# This is not sent between machines, just used when seeing what type of Ack to expect
def NoAck():
    return "NoAck"

def Data():
    return "Data"

def Ack():
    return "Ack"

def Retrieve():
    return "Retrieve"

def Fail():
    return "Fail"

# for case when local scrubber has detected some bitrot and asks customer to resend
# def Resend():
#    return("Resend")

def ListFiles():
    return "ListFiles"

def Files():
    return "Files"

def ListContacts():
    return "ListContacts"

def Contacts():
    return "Contacts"

def NearnessCheck():
    return "NearnessCheck"

def Nearness():
    return "Nearness"

def RequestIdentity():
    return "RequestIdentity"

def Identity():
    return "Identity"

def DeleteFile():
    return "DeleteFile"

def DeleteBackup():
    return "DeleteBackup"

def Transfer():
    return "Transfer"

def Receipt():
    return "Receipt"

def Message():
    return "Message"

def Correspondent():
    return "Correspondent"

#  Next commands are for talking to Central
def initCentralCommands():
    return

# check to see if is a valid central command
def IsCentralCommand(s):
    result=False
    if (s==Ack()):result=True
    if (s==Register()):result=True
    if (s==Identity()):result=True
    if (s==Suppliers()):result=True
    if (s==Customers()):result=True
    if (s==FireContact()):result=True
    if (s==Settings()):result=True
    if (s==Transfer()):result=True
    if (s==BandwidthReport()):result=True
    if (s==RequestSuppliers()):result=True
    if (s==RequestCustomers()):result=True
    if (s==RequestSpace()):result=True
    if (s==RequestSettings()):result=True
    if (s==RequestReceipt()):result=True
    return(result)

def Register():
    return("Register")

def RequestSuppliers():
    return("RequestSuppliers")

def Suppliers():
    return("Suppliers")

def RequestCustomers():
    return("RequestCustomers")

def Customers():
    return("Customers")

def FireContact():
    return("FireContact")

def Settings():
    return('Settings')

def BandwidthReport():
    return 'BandwidthReport'

def RequestSpace():
    return 'RequestSpace'

def RequestSettings():
    return 'RequestSettings'

def RequestReceipt():
    return "RequestReceipt"






#!/usr/bin/python
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
# SUPPLIERPATROL.PY
#
# We patrol the suppliers we have and make sure they are all working well, or we work
# out the problems.  Working out problems may be fixing one missing packet, or
# replacing a supplier and rebuilding all the data he had (or not).
#
# We keep on disk the most recent "ListFiles" from each supplier.
# We can look for any missing packets in the list (maybe deleted from bitrot or never got through).
#
# Different tests we can do to check that remote nodes are working the way we want them to.
#
#   1)  We ask them to "listfiles" and see that they all give the same answers (unless in middle of backup)
#   2)  We can read random remote dhnpackets and check digital signatures.
#       This is very simple, and probably all we really need to do.
#   3)  We can use one parity and the data packets for it and check that parity works.
#          If parity does not work, we have at least one bad node.
#          Should really have shown up in signature test already (done as part of reading packets).
#
#          There is a chance that a header was corrupted.  Could use other parities to check which
#          packet is causing problem.
#   4)  We can just ask them to send their identity and see that they do (sort of ping)
#
# We also keep track of bandwidth we got from different sites.
# Information we collect helps "fire_hire" decide if some supplier needs to be fired.
#
# We need to know BackupIDs and max BlockNumber so we can fetch in right range.

import os
import time


#from twisted.internet.defer import Deferred


import lib.dhnio as dhnio
import lib.misc as misc
import lib.settings as settings
import lib.contacts as contacts
import lib.transport_control as transport_control

import lib.commands as commands
import lib.dhnpacket as dhnpacket


#-------------------------------------------------------------------------------


def fileAgeInSeconds(filename):
    now=time.time()
    filemtime = os.path.getmtime(filename)
    age = now - filemtime
    if (age<0):
        age=0
    return(age)


NumRequestsOutstanding=0
TestDeferred=""

def ListResult(packet):
    dhnio.Dprint(7, "supplierpatrol.ListResult got result from " + packet.OwnerID)
    supnum=contacts.numberForSupplier(packet.OwnerID)
    filename= os.path.join(settings.FileListDir(), str(supnum))
    dhnio.WriteFile(filename, packet.Data)
    global NumRequestsOutstanding
    NumRequestsOutstanding -= 1
    dhnio.Dprint(7, "supplierpatrol.ListResult  NumRequestsOutstanding = " + str(NumRequestsOutstanding))
    CheckCallback()

def CheckCallback():
    global TestDeferred
    global DataResultsOutstanding
    if (NumRequestsOutstanding == 0 and len(DataResultsOutstanding)==0 and TestDeferred != ""):
        TestDeferred.callback(1)

# We refetch any ListFiles result that is too old, or maybe fire a supplier.
def UpdateListFiles():
    if (not os.path.exists(settings.FileListDir())):
        os.mkdir(settings.FileListDir())
    for supnum in range(0, contacts.numSuppliers()):
        filename= os.path.join(settings.FileListDir(), str(supnum))
        dhnio.Dprint(7, "supplierpatrol.UpdateListFiles  looking at = " + filename)
        if (not os.path.exists(filename) or (fileAgeInSeconds(filename) > 3600*24)):
            dhnio.Dprint(7, "supplierpatrol.UpdateListFiles  found one to update " + filename)
            command=commands.ListFiles()
            OwnerID=misc.getLocalID()
            CreatorID=misc.getLocalID()
            PacketID="ListFiles" + str(supnum)
            Payload=""
            RemoteID= contacts.getSupplierID(supnum)
            request=dhnpacket.dhnpacket(command, OwnerID, CreatorID, PacketID, Payload, RemoteID)
            transport_control.RegisterInterest(ListResult, RemoteID, PacketID)
            transport_control.outboxAck(request)
            global NumRequestsOutstanding
            NumRequestsOutstanding += 1
            dhnio.Dprint(7, "supplierpatrol.UpdateListFiles  sent request - now outstanding=" + str(NumRequestsOutstanding))


DataResultsOutstanding=[]

def DataResult(packet):
    dhnio.Dprint(7, "supplierpatrol.DataResult  got" + packet.PacketID)
    global DataResultsOutstanding
    DataResultsOutstanding.remove(packet.PacketID)
    CheckCallback()

#
def OneFromList(filename):
    WholeFile=dhnio.ReadBinaryFile(filename)
    FileList=WholeFile.split("\n")
    num=len(FileList)
    dhnio.Dprint(7, "supplierpatrol.OneFromList  number of items is " + str(num))
    rnd=random.randint(0,num-1)
    item=FileList[rnd]
    command=commands.Data()
    OwnerID=misc.getLocalID()
    CreatorID=misc.getLocalID()
    PacketID=item
    Payload=""
    supnum=packetid.SupplierNumber(item)
    RemoteID=contacts.getSupplierID(supnum)
    request=dhnpacket.dhnpacket(command, OwnerID, CreatorID, PacketID, Payload, RemoteID)
    global DataResultsOutstanding
    DataResultsOutstanding.append(item)
    transport_control.RegisterInterest(DataResult, RemoteID, PacketID)
    transport_control.outboxAck(request)

# Randomly test read a file.  Need to see that we really get it and
# that it has a good checksum.  For now we ask for 1 from each supplier.
# We just work off the files they say they have.
def RandomSample():
    for supnum in range(0, contacts.numSuppliers()):
        filename= os.path.join(settings.FileListDir(), str(supnum))
        if (os.path.exists(filename)):
            OneFromList(filename)



# REBUILDRANGE - range of packets for a node (probably only 1 or all for node)
#  Most common case will probably be after we replace a node, but sometimes
#  this will be used to repair bitrot damage.
#
# To rebuild we will read several data and one parity packet.  We first
# find minimal size parity to rebuild and do the rebuild.  We might have some
# cache of packets, but probably we need to fetch all data and parity we will use.
# So we probably want a parity with the smallest number of components that
# includes the one we are trying to fix.
#
# There may be times when there are several broken nodes and we would like
# to find the set of parities with the most nodes in common to reduce the
# total number of reads.
#
# Note also that the plan may need to change partway through as some
# new node failure may occure.
#
# Danger is that failures come in faster than we can fix them.  If this is
# happening, we need a plan-B, which is to call a Professional Scrubber.
# those guys have plenty of bandwidth and can get your data cleaned up fast.
#
# PREPRO

##class supplierpatroltest(unittest.TestCase):
##    def test_supplierpatrol(self):
##        dhninit.init()
##        UpdateListFiles()
##        global TestDeferred
##        TestDeferred=Deferred()
##        return(TestDeferred)




#!/usr/bin/python
#schedule.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import time
import calendar

if __name__ == '__main__':
    sys.path.append(os.path.abspath('..'))

import lib.dhnio as dhnio
import lib.misc as misc
import lib.dhnmath as dhnmath

#------------------------------------------------------------------------------ 

all_types = {
    '0': 'none',
    '1': 'hourly',
    '2': 'daily',
    '3': 'weekly',
    '4': 'monthly',
    '5': 'continuously'}

all_labels = {
    'n': 'none',
    'h': 'hourly',
    'd': 'daily',
    'w': 'weekly',
    'm': 'monthly',
    'c': 'continuously',}

class Schedule:
    types = all_types
    labels = all_labels
    
    def __init__(self, 
                 typ = None, 
                 daytime = None, 
                 interval = None, 
                 details = None, 
                 lasttime = None,
                 from_tupple = None,
                 from_string = None,
                 from_dict = None,
                 ):
        self.type = str(typ)
        if self.type in ['0', '1', '2', '3', '4', '5']:
            self.type = self.types.get(self.type, 'none')
        self.daytime = str(daytime)
        self.interval = str(interval)
        self.details = str(details)
        self.lasttime = ''
        if from_string is not None:
            self.from_dict(self.correct_dict(self.string_to_dict(from_string)))
        if from_tupple is not None:
            self.from_dict(self.correct_dict(self.tpple_to_dict(from_tupple)))
        if from_dict is not None:
            self.from_dict(self.correct_dict(from_dict))
        if 'None' in [ self.daytime, self.interval, self.details ]:
            if self.type is None:
                self.from_dict(self.blank_dict('none'))
            else:
                self.from_dict(self.blank_dict(self.type))
    
    def __repr__(self):
        return 'Schedule(%s, %s, %s, %s, %s)' % (
            self.type, self.daytime, self.interval, self.details, self.lasttime)        
    
    def blank_dict(self, type):
        d = { 'type': type }
        if type == 'none':
            d['interval'] = ''
            d['daytime'] = ''
            d['details'] = ''
            d['lasttime'] = ''
        elif type == 'continuously':
            d['interval'] = '600'
            d['daytime'] = ''
            d['details'] = ''
            d['lasttime'] = ''
        elif type == 'hourly':
            d['interval'] = '1'
            d['daytime'] = ''
            d['details'] = ''
            d['lasttime'] = ''
        elif type == 'daily':
            d['interval'] = '1'
            d['daytime'] = '12:00:00'
            d['details'] = ''
            d['lasttime'] = ''
        elif type == 'weekly':
            d['interval'] = '1'
            d['daytime'] = '12:00:00'
            d['details'] = 'Monday Tuesday Wednesday Thursday Friday Saturday Sunday'
            d['lasttime'] = ''
        elif type == 'monthly':
            d['interval'] = '1'
            d['daytime'] = '12:00:00'
            #d['details'] = 'January February March April May June July August September October November December'
            d['details'] = '1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31'
            d['lasttime'] = ''
        return d
    
    def string_to_dict(self, raw_data):
        l = raw_data.split('\n')
        typ = l[0].strip()
        if typ in ['0', '1', '2', '3', '4', '5']:
            typ = self.types.get(typ, 'none')
        if typ not in self.types.values():
            typ = 'none'
        if len(l) < 3:
            return self.blank_dict(typ)
        return self.tpple_to_dict(l)
    
    def tpple_to_dict(self, tuppl):
        d = {}
        d['type'] = str(tuppl[0]).strip()
        d['daytime'] = str(tuppl[1] if len(tuppl) > 1 else '12:00:00').strip()
        d['interval'] = str(tuppl[2] if len(tuppl) > 2 else '1').strip()
        d['details'] = str(tuppl[3] if len(tuppl) > 3 else '').strip()
        d['lasttime'] = str(tuppl[4] if len(tuppl) > 4 else '').strip()
        return d
    
    def from_dict(self, d):
        self.type = d.get('type', 'none')
        self.daytime = d.get('daytime', '12:00:00')
        self.interval = d.get('interval', '1')
        self.details = d.get('details', '')
        self.lasttime = d.get('lasttime', '')
    
    def to_dict(self):
        d = {}
        d['type'] = self.type
        d['daytime'] = self.daytime
        d['interval'] = self.interval
        d['details'] = self.details
        d['lasttime'] = self.lasttime
        return d
    
    def to_string(self):
        s = ''
        s += self.type + '\n'
        s += self.daytime + '\n'
        s += self.interval + '\n'
        s += self.details + '\n'
        s += self.lasttime + '\n'
        return s
    
    def next_time(self):
        lasttime = self.lasttime
        if lasttime == '':
            # let it be one year ago (we can schedule 1 month maximum) and one day
            lasttime = str(time.time() - 366*24*60*60)
    
        try:
            # turned off - return -1
            if self.type in ['none', 'disabled']:
                return -1
            
            # every N seconds
            elif self.type == 'continuously':
                return dhnmath.shedule_continuously(lasttime, int(self.interval),)
        
            # every N hours, exactly when hour begins, minutes and seconds are 0
            elif self.type == 'hourly':
                return dhnmath.shedule_next_hourly(lasttime, int(self.interval),)
        
            # every N days, at given time
            elif self.type == 'daily':
                return dhnmath.shedule_next_daily(lasttime, self.interval, self.daytime)
        
            # every N weeks, at given time and selected week days
            elif self.type == 'weekly':
                week_days = self.details.split(' ')
                week_day_numbers = []
                week_day_names = list(calendar.day_name)
                for week_label in week_days:
                    try:
                        i = week_day_names.index(week_label)
                    except:
                        continue
                    week_day_numbers.append(i)
                return dhnmath.shedule_next_weekly(lasttime, self.interval, self.daytime, week_day_numbers)
        
            # monthly, at given time and day 
            elif self.type == 'monthly':
                month_dates = self.details.split(' ')
                return dhnmath.shedule_next_monthly(lasttime, self.interval, self.daytime, month_dates)

            # yearly, at given time and month, day, NOT DONE YET! 
            elif self.type == 'yearly':
                months_labels = self.details.split(' ')
                months_numbers = []
                months_names = list(calendar.month_name)
                for month_label in months_labels:
                    try:
                        i = months_names.index(month_label)
                    except:
                        continue
                    months_numbers.append(i)
                return dhnmath.shedule_next_monthly(lasttime, self.interval, self.daytime, months_numbers)
        
            else:
                dhnio.Dprint(1, 'schedule.next_time ERROR wrong schedule type: ' + self.type)
                return None
        except:
            dhnio.DprintException()
            return None 

    def correct_dict(self, d):
        if not d.has_key('type'):
            d['type'] = 'none'
        if d['type'] in ['0', '1', '2', '3', '4', '5']:
            d['type'] = self.types.get(d['type'], 'none')
        if not d.has_key('daytime'):
            d['daytime'] = '12:00:00'
        if not d.has_key('interval'):
            d['interval'] = '1'
        if not d.has_key('details'):
            d['details'] = ''
        if not d.has_key('lasttime'):
            d['lasttime'] = ''
        if d['type'] not in self.types.values():
            d['type'] = self.types['0']
        try:
            d['interval'] = str(int(d['interval']))
        except:
            d['interval'] = '1'
        if d['daytime'] == '' or d['daytime'] is None or d['daytime'] == 'None':
            d['daytime'] = time.strftime('%H:%M:%S', time.localtime())
        time_parts = d['daytime'].split(':')
        if len(time_parts) == 1:
            d['daytime'] = misc.DigitsOnly(d['daytime'])
            if int(time_parts[0]) > 24:
                time_parts[0] = '0'
            time_parts.append('00')
        elif len(time_parts) > 1:
            time_parts[0] = misc.DigitsOnly(time_parts[0])
            time_parts[1] = misc.DigitsOnly(time_parts[1])
            if int(time_parts[0]) > 24:
                time_parts[0] = '0'
            if int(time_parts[1]) > 59:
                time_parts[1] = '00'
            if len(time_parts) > 2:
                time_parts[2] = misc.DigitsOnly(time_parts[2])
                if int(time_parts[2]) > 59:
                    time_parts[2] = '00'
        if len(time_parts) < 3:
            time_parts.append('00')
        d['daytime'] = '%02d:%02d:%02d' % (int(time_parts[0]), int(time_parts[1]), int(time_parts[2]))
        if d['type'] == 'weekly' and d['details'].strip() == '':
            d['details'] = 'Monday Tuesday Wednesday Thursday Friday Saturday Sunday'
        if d['type'] == 'monthly' and d['details'].strip() == '':
            d['details'] = 'January February March April May June July August September October November December'
        return d
    
    def description(self):
        if self.type == 'none':
            return 'not scheduled'
        if self.type == 'hourly':
            if self.interval == '1':
                return 'every hour'
            else:
                return 'every %s hours' % self.interval
        if self.type == 'continuously':
            return 'every %s seconds' % self.interval
        if self.type == 'daily':
            if self.interval == '1':
                return 'every day, at %s' % self.daytime
            else:
                return 'every %s days, at %s' % (self.interval, self.daytime)
        if self.type == 'weekly':
            if self.interval == '1':
                return 'every week, at %s, in %s' % (
                    self.daytime, self.details.strip().replace(' ', ','))
            else:
                return 'every %s weeks, at %s, in %s' % (
                    self.interval, self.daytime, self.details.strip().replace(' ', ', '))               
        if self.type == 'monthly':
            return 'in day %s of %s, at %s' % (
                self.interval, self.details.strip().replace(' ', ', '), self.daytime)
        return 'incorrect schedule type'
 
    def html_description(self):
        if self.type == 'none':
            return 'not scheduled'
        if self.type == 'hourly':
            if self.interval == '1':
                return 'every hour'
            else:
                return 'every <b>%s</b> hours' % self.interval
        if self.type == 'continuously':
            return 'every <b>%s</b> seconds' % self.interval
        if self.type == 'daily':
            if self.interval == '1':
                return 'every day, at <b>%s</b>' % self.daytime
            else:
                return 'every <b>%s</b> days, at <b>%s</b>' % (self.interval, self.daytime)
        if self.type == 'weekly':
            if self.interval == '1':
                return 'every week, at <b>%s</b>, in <b>%s</b>' % (
                    self.daytime, self.details.strip().replace(' ', ','))
            else:
                return 'every <b>%s</b> weeks, at <b>%s</b>, in <b>%s</b>' % (
                    self.interval, self.daytime, self.details.strip().replace(' ', ', '))               
        if self.type == 'monthly':
            return 'in <b>%s</b> dates of every <b>%s</b> month, at <b>%s</b>' % (
                self.details.strip().replace(' ', ', '), self.interval, self.daytime)
        return 'incorrect schedule type'
        
    def html_next_start(self):
        next = self.next_time()
        if next is None:
            return ''
        if next < 0:
            return ''
        try:
            # nextString = time.asctime(time.localtime(next))
            nextString = time.strftime('%A, %d %B %Y %H:%M:%S', time.localtime(next))
        except:
            dhnio.DprintException()
            return ''
        return 'next execution expected at <b>%s</b>' % nextString
        
#------------------------------------------------------------------------------ 

def format():
    return '''
Schedule compact format:
[mode].[interval].[time].[details]

mode:
  n-none, h-hourly, d-daily, w-weekly, m-monthly, c-continuously
  
interval:
  just a number - how often to restart the task, default is 1
    
time:
  [hour]:[minute]
  
details:
  for weeks: Mon Tue Wed Thu Fri Sat Sun
  for months: Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec
  
some examples:
  n.1..                   no schedule
  hourly.3                each 3 hours, for example 01:00:00, 04:00:00, 07:00:00, ...
  daily.4.10:15.          every 4th day at 10:15
  w.1.3:00.MonSat         every Monday and Saturday at 3:00 in the night
  weekly.4.18:45.MonTueWedThuFriSatSun
                          every day in each 4th week in 18:45
  m.5.12:34.JanJul        5th Jan and 5th July at 12:34
  c.300                   every 300 seconds (10 minutes)
'''

    
def default_dict():
    return {   'type':      'daily',
               'daytime':   '12:00:00',
               'interval':  '1',
               'details':   '',
               'lasttime':  ''}
    
def default():
    return Schedule(from_dict=default_dict())

def empty():
    return Schedule('none', '', '', '')
    
#------------------------------------------------------------------------------ 

def from_compact_string(s):
    try:
        parts = s.split('.')
        parts += ['']*(4-len(parts))
        (sh_type, sh_interval, sh_time, sh_details) = parts[0:4]
        sh_type = sh_type.lower()
        try:
            sh_interval = int(sh_interval)
        except:
            sh_interval = 1
        if sh_type not in ['n', 'none', 'h', 'hourly', 'c', 'continuously']:
            try:
                time.strptime(sh_time, '%H:%M')
            except:
                try:
                    time.strptime(sh_time, '%H:%M:%S')
                except:
                    dhnio.Dprint(2, 'schedule.compact_string_to_dict WARNING incorrect schedule time: ' + s)
                    dhnio.DprintException()
                    return None
    except:
        dhnio.Dprint(2, 'schedule.compact_string_to_dict WARNING incorrect schedule string: ' + s)
        dhnio.DprintException()
        return None
    if sh_type in all_labels.keys():
        sh_type = all_labels[sh_type]
    if sh_type not in all_labels.values():
        dhnio.Dprint(2, 'schedule.compact_string_to_dict WARNING incorrect schedule type: ' + s)
        return None
    sh_details_new = ''
    for i in range(len(sh_details) / 3):
        label = sh_details[i*3:i*3+3]
        if sh_type == 'weekly' and not label in calendar.day_abbr:
            dhnio.Dprint(2, 'schedule.compact_string_to_dict WARNING incorrect schedule details: ' + s)
            return None
        if sh_type == 'monthly' and not label in calendar.month_abbr:
            dhnio.Dprint(2, 'schedule.compact_string_to_dict WARNING incorrect schedule details: ' + s)
            return None
        sh_details_new += label + ' '
    return Schedule(sh_type, str(sh_time), str(sh_interval), sh_details_new.strip())   


    